/***************************************************************************
                          guibalances.cpp  -  description
                             -------------------
    begin                : Do Nov 4 2004
    copyright            : (C) 2004 by Marcel Mulch
    email                : marcel@mulch-online.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "guibalances.h"
#include <openhbci2/value.h>
#include "aqmoney2/account.h"
#include <openhbci2/date.h>
#include <qdatetime.h>


//layout
#include <qlayout.h>
#include <qlabel.h>
#include <qpushbutton.h>


/**
* This is constructor of class for showing balances of all listed accounts
* @param main
* @param parent the parent dialog of this widget
* @param name widgets title
* @param hobit
*/
GuiBalances::GuiBalances(GuiMain *main, QWidgetStack *parent, const char *name, HoBIT* hobit) :
    GuiBalancesBase(parent,name)
{
  _main = main;
  _hobit = hobit;
  updateList();

  setTabOrder(listViewAccounts, pushButtonAccountReload);
}

GuiBalances::~GuiBalances()
{ 
  qDebug("Destructor GuiBalances called");
  //delete layout;
  qDebug("Destructor GuiBalances finished");
}

/**
* Update List of Balances
*/
bool GuiBalances::updateList()
{

  listViewAccounts->clear();

  list<Pointer<Account> > acclist = _hobit.ref().listAccounts();

  if (acclist.empty())
    new QListViewItem(listViewAccounts, "Keine Eintr�e", "","");

  else
  {

    list<Pointer<Account> > :: iterator accit;

    for (accit = acclist.begin(); accit != acclist.end(); accit++)
    {

      QString accountId = (*accit).ref().accountId().c_str();
      QString balance, n;
      long val = (long) (100* (*accit).ref().bookedBalance().value().getValue());
      n.setNum((double)(val - val%100) / 100.00);
      balance = n + ",";
      n.setNum(val%100);
      balance += n + " ";
      balance += (*accit).ref().bookedBalance().value().getCurrency().c_str();
      Date d = (*accit).ref().bookedBalance().date();
      QDate date(d.year(), d.month(), d.day());

      new QListViewItem(listViewAccounts, accountId, balance, date.toString());

    } // end for

  } // end else

  return true;
}

/**
*retrieves the balances of all accounts, if chosen
*/
void GuiBalances::slotGetBalances()
{
  _hobit.ref().createJobGetBalance();
  if( _main->askForSend())
    _main->slotSendNow();
}


/*!
    \fn GuiBalances::doLayout()
 */

/**
* set the widgets base layout. will be called 
* from the constructor only
*/
void GuiBalances::doLayout()
{
  layout = new QGridLayout(this,			//parent
                           3, 			//rows
                           3,			//columns
                           0,			// margin
                           6,			//space
                           "layout");		//name


  QSpacerItem *spacer1 = new QSpacerItem(20,20, QSizePolicy::Fixed, QSizePolicy::Minimum);
  QSpacerItem *spacer2 = new QSpacerItem(20,20, QSizePolicy::Fixed, QSizePolicy::Minimum);


  layout->setResizeMode(QLayout::FreeResize);

  layout->addWidget(textLabelName,0,1);
  layout->addItem(spacer1,1,0);
  layout->addWidget(listViewAccounts,1,1);
  layout->addItem(spacer2,1,2);
  layout->addWidget(pushButtonAccountReload,2,1);


  update();
}


/*!
    \fn GuiBalances::refresh()
 */
/**
*Update List of Balances
*/
void GuiBalances::refresh()
{
  updateList();
}
