/***************************************************************************
 *   Copyright (C) 2005 by Marcel Mulch                                    *
 *   marcel@linux                                                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "guiburn.h"
#include "globals.h"

#include <qcombobox.h>
#include <qlabel.h>
#include <qpushbutton.h>
#include <qprogressbar.h>
#include <qstringlist.h>

GuiBurn::GuiBurn(QWidget *parent, const char *name)
 : GuiBurnBase(parent, name)
{
	_burnTimer = new QTimer( this );
        connect( _burnTimer, SIGNAL(timeout()), this, SLOT(refreshStatus()) );
        //timer->start( 2000, TRUE ); // 2 seconds single-shot timer
	
	buttonStart->setEnabled(true);
  
}


GuiBurn::~GuiBurn()
{
qDebug("Destructor GuiBurn finished");
}

void GuiBurn::accept()
{
  _burnProcess = new QProcess(this);
 
  labelStatus->setBackgroundColor(Qt::green);  
  labelStatus->setText("burning started");
 
  _burnTimer->start(1000, false);
  system("cdrecord -v dev=/dev/cdrecorder speed="+comboBoxSpeed->currentText() 
          +" ~/moneypenny-" + VERSION + ".iso > /tmp/moneypennyburn.tmp &");
  buttonStart->setEnabled(false);
  progressBar->setProgress(0, 100);
}



void GuiBurn::refreshStatus()
{
  system("tail --lines=1 /tmp/moneypennyburn.tmp > /tmp/moneypennyburn.tmp2");
  _tempfile = new QFile("/tmp/moneypennyburn.tmp2");
  
    if ( _tempfile->open( IO_ReadOnly ) ) {
        QTextStream stream( _tempfile );
	
	QString stat;
	
       while ( !stream.atEnd() )
		stat = stream.readLine(); //get last line
	
	if (stat.startsWith("Last chance to quit")) {
		stat = tr("preparing burn process");
		labelStatus->setBackgroundColor(Qt::yellow);	
	}
		
		
	if (stat.startsWith("Track")) {
	
	
         QStringList sl = QStringList::split(" ", stat);
	 	
	  QString done = *sl.at(2);
	  QString full = *sl.at(4);
	  
	  int mdone = done.toInt();
	  int mfull = full.toInt();
	 
	  progressBar->setProgress(mdone, mfull);
	}
	
	if (stat.startsWith("Lout start:") && stat.endsWith("x.")) {
          stat = tr("Please insert a writable medium first");
          _burnTimer->stop();
          _burnProcess->kill();
           labelStatus->setBackgroundColor(Qt::red);
           buttonStart->setEnabled(true);
	};		
			
		
	if (stat.startsWith("Re-load disk")) {
          stat = tr("bad Disk");
          _burnTimer->stop();
          _burnProcess->kill();
           labelStatus->setBackgroundColor(Qt::red);
           buttonStart->setEnabled(true);
	};
	
        labelStatus->setText(stat);
	
	if (stat.startsWith("Fixating time")) {
           stat = tr("Live-CD has been written successfully");
           progressBar->setProgress(100, 100);
           labelStatus->setText(stat);
           labelStatus->setBackgroundColor(Qt::green);
           _burnTimer->stop();
           system("eject cdrom");
           buttonStart->setEnabled(true);
           _burnTimer->stop();
	}
       
    }
   
   _tempfile->close(); 
}


void GuiBurn::stop()
{
	QString stat = "stopped by user";
	_burnTimer->stop();
	_burnProcess->kill();
	labelStatus->setBackgroundColor(Qt::red);
	        labelStatus->setText(stat);
	buttonStart->setEnabled(true);
}
   



