/***************************************************************************
 *   Copyright (C) 2004 by Walter Roth                                     *
 *   walter@amd2600-roth                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef GUIHARDWARECONFIG_H
#define GUIHARDWARECONFIG_H

#include "guihardwareconfigbase.h"
#include <qprocess.h>
#include "guimain.h"

class GuiMain;

/**
* class for show hardware-configuration of modem, isdn, dsl, lan, printer, home and save
* starts the Knoppix hardware configuration tools.
*/
class GuiHardwareConfig: public GuiHardwareConfigBase
{
  Q_OBJECT
public:
  /**
  *this is the constructor of class for hardware configuration 
  *
  *@param main 
  *@param parent the parent dialog of this widget
  *@param hobit
  */
  GuiHardwareConfig(QWidget * parent, const char *name = 0);

/** destructor*/
~GuiHardwareConfig();



public slots:
  /** These functions call external KNOPPIX configuration programs
  */
  virtual void modemconfig();
  virtual void isdnconfig();
  virtual void lanconfig();
  virtual void dslconfig();
  virtual void wlanconfig();
  virtual void gprsconfig();
  virtual void saveconfig();
  /** Emits signal printerAutoSetupRequest
  */
  virtual void printerautosetup();
  /** Emits signal printerManualSetupRequest
  */
  virtual void printermanualsetup();
  /** Emits signal finished
  */
  virtual void accept();
  /** Copy the predefined kppprc file to ~/.kde/share/config
  */
  void slotTryPredefinedModem();
  /** Copy the predefined isdn file to /etc/sysconfig/isdn
  */
  void slotTryPredefinedISDN();

signals:

  void finished();
  void raiseWidget(int);
private:
 /** Copies file from src to dest using root privileges. For use with KNOPPIX only.
 * Uses sudoe if asRoot is true.
 */
 bool copyConfigFile(QString srcPath, QString destPath, QString fileName, bool asRoot);
 
 // process to be executed
  QProcess* _Process;    
};

#endif
