/***************************************************************************
 *   Copyright (C) 2004 by Marcel Mulch                                    *
 *   marcel@mulch-online.de                                                *
 *                                                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef GUIJOBS_H
#define GUIJOBS_H

#include "guijobsbase.h"
#include "guimain.h"
#include "hobit.h"

class GuiMain;

/**
*Class for the GuiJobs
*/
class GuiJobs: public GuiJobsBase
{

  Q_OBJECT

public:

  /**
  *constructor of GuiJobs
  *
  *@param main a pointer on moneypenny´s main class
  *@param parent a pointer on the main widgets widget stack
  *@param name widgets title
  *@param hobit the hobit object, needed for hbci functions
  */
  GuiJobs(QWidget *parent, GuiMain* guimain, const char *name = 0, HoBIT* hobit = 0);

  /** destructor */
  ~GuiJobs();

  /**
  *refreshes the job queues 
  */
  virtual void refresh();

  /**
  *analyzing of the account job
  *@param job pointer on AccountJob
  */
  void analyzeAccountJob(Pointer<AccountJob> job);

  /**
  *analyzing of the admin job
  *@param job pointer on AdminJob
  */
  void analyzeAdminJob(Pointer<AdminJob> job);

/** set hobit object*/
  void setHoBIT(HoBIT* hobit){
   _hobit = hobit;
  }

public slots:

  /**
  *sets the selected account jobs
  */
  virtual void slotSelectAccountJob();

  /**
  *sets the selected admin jobs
  */
  virtual void slotSelectAdminJob();

  /**
  *slot for deleting jobs
  */
  virtual void slotDeleteJob();

  /** 
  *Deletes all jobs
  */
  void slotDeleteAllJobs();
  
  /**
  *sends the jobs when executes
  */
  virtual void slotSendNow();

private:

  GuiMain* _main;
  HoBIT* _hobit;
  int _count;
  bool _adminSelected;
  bool _accountSelected;
};

#endif
