/***************************************************************************
 *   Copyright (C) 2004 by Walter Roth                                     *
 *   walter@amd2600-roth                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "guiloginpassword.h"
#include "string.h"
#include <qapplication.h>
#include <qstring.h>
#include <qpushbutton.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qfile.h>
#include <qcursor.h>
#include <qmessagebox.h>
#include <qlayout.h>

#include "hobit_crypt.h"

/**
*constructor of GuiLoginPassword
*
*@param app pointer on QApplication
*@param homedir name of home-directory
*@param crypt a pointer on the HoBIT_Crypt Interface
*/
GuiLoginPassword::GuiLoginPassword(QWidget * parent, HoBIT_Crypt * crypt)
 : GuiLoginPasswordBase(parent){
   _crypt = crypt;
   _CreateNewPassword = true;
   lineEditPassword->setText("");
   lineEditPassword2->setText("");
}


/**
*destructor of GuiLoginPassword
*/
GuiLoginPassword::~GuiLoginPassword()
{
  qDebug("Destructor GuiLoginPassword finished");
}

/**
*slot which executes when no crypfile exists
*/
void GuiLoginPassword::slotLogin(){
 int passwordState =0;
 
  string pw = lineEditPassword->text().ascii();
  
  if (_CreateNewPassword) {// no cryptfiles exists, write a new one  
      if(lineEditPassword->text() !=lineEditPassword2->text()){
        QMessageBox::critical(this, tr("Password Error"), tr("The passwords you input are not identical."));
        return;
      }
     _crypt->setPassword(pw);     
      passwordState = _crypt->savePasswordMD();              
      if (passwordState != 0) 
        QMessageBox::critical(this, tr("Encryption Error"), tr("Can not save hash value."));     
  }
  else{ //check password
     _crypt->setPassword(pw);     
     passwordState =  _crypt->checkPasswordMD("");
     if (passwordState == -2) //invalid password
       QMessageBox::information(this, tr("Password Error"), tr("The password you input is invalid."));
  }//end check password
    
  if (passwordState == 0) 
   accept();     
}

/**
*slot for closing
*/
void GuiLoginPassword::slotClose(){
  setCursor(QCursor::WaitCursor);
  reject();
}

/**
*slot for the internet setup
*/
void GuiLoginPassword::slotInternetSetup(){
 QMessageBox::information(this, tr("Sorry"),tr("Not implemented yet"));	
}

/**
*shows a short help message in a mesagebox.
*/
void GuiLoginPassword::slotHelp(){
   QString msg =
   tr("If there is a single edit field, enter your password in this field and press 'Return'. \
       If there are two edits, enter an identical new password into both edits. \
      <br>More help is available from the 'Help' button on top of the main window of MoneyPenny. \
       It will pop up a short time after your password has been checked.");
   QMessageBox::information(this, "MoneyPenny Help", msg);
}
  
/** Checks for an existing password file
*/
bool GuiLoginPassword::checkForPasswordFile(){
   lineEditPassword->setText("");
   lineEditPassword2->setText("");

   QFile pwFile(_crypt->getCryptDir() + "/" + MONEYPENNY_PASSWORD_FILE);
      
   if (pwFile.exists()) {
     _CreateNewPassword = false;
     lineEditPassword2->hide();
     textLabelPassword2->hide();
     textLabelFileDetected->setText(tr("Your password verification file has been found."));
   }
   else  {
    _CreateNewPassword = true;
     lineEditPassword2->show();
     textLabelPassword2->show();
   } 
   //now that all components are in their right visibilty state, resize and reposition the widget.
   adjustSize();
   return _CreateNewPassword; 
}

/** Prepares dialog for a password change
*/
void GuiLoginPassword::switchToEditPasswordMode(){
     lineEditPassword->setText("");
     lineEditPassword2->setText("");
     lineEditPassword2->show();
     textLabelPassword2->show();
     textLabelFileDetected->setText(tr("Please input your new password into both edit fields."));
     adjustSize();
     _CreateNewPassword = true;
}

/**Resizes accoprding to visible widgets
*/
void GuiLoginPassword::adjustSize(){
     setGeometry(x(),y(), width(), 200); //this is too narrow, force widgets to shrink
     qApp->processEvents(); // now resize everything to minimum
     int newHeight = buttonLogin->y() + buttonLogin->height()+5;
     int newY = (qApp->desktop()->height() - newHeight) /2;
     setGeometry(x(),newY, width(), newHeight); //this is the proper height.
}
