/***************************************************************************
                          guimain.cpp  -  description
                             -------------------
    begin                : Do Nov 4 2004
    copyright            : (C) 2004 by Marcel Mulch
    email                : marcel@mulch-online.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "guimain.h"
#include "guitools.h"
#include "guishowdata.h"
#include "guiwelcome.h"
#include "guibalances.h"
#include "guitransactions.h"
#include "guigettransactions.h"
#include "guiiniletter.h"
#include "guitransfertemplates.h"
#include "guisingletransfer.h"
#include "guidatedsingletransfer.h"
#include "guidatedtransfers.h"
#include "guistandingorders.h"
#include "guistartstandingorder.h"
#include "guitransactionprinter.h"
#include "guihelpwindow.h"
#include "guiknownbanks.h"
#include "guilogin.h"
#include "guilog.h"
#include "guiaccountinformation.h"
#include "guihbciwizard.h"
#include "guichangeaccountname.h"
#include "guiaddaccount.h"
#include "guijobs.h"
#include "guihardwareconfig.h"
#include "guidialin.h"
#include "guifindmedium.h"
#include "guiloginpassword.h"
#include "guidataexport.h"
#include "guisqlquery.h"
#include "guiburn.h"	           // delete after cebit	
#include "guisplash.h"
#include "guitransactioninfo.h"
#include "guisqlqueryresult.h"
#include "guiprinterautosetup.h"
#include "guiprintermanualsetup.h"
#include "guiprinterdrivertoxml.h"


#include "hobit.h"
#include "hobit_connector.h"
#include "hobit_crypt.h"
#include "hobit_sql.h"
#include "listviewitems.h"

#include "keyfileopenhbci.h" 


#include <qlayout.h>
#include <qlabel.h>
#include <qpushbutton.h>
#include <qwidget.h>
#include <qmenubar.h>
#include <qstringlist.h>
#include <qcursor.h>
#include <qapplication.h>
#include <qfiledialog.h>
#include <qinputdialog.h>


/**
*constructor for GuiMain (main window),
*sets statusbar, menulist, refreshes informations, widgets, slots, etc. 
*@param crypt pointer on the HoBIT_Crypt interface
*@param parent pointer on current QWidget
*@param name title of the GuiMain dialog
*/
GuiMain::GuiMain() : GuiMainBase(NULL,"guiMain")
{ 
  qDebug("Constructor GuiMain starts");
  
  setCaption(QString("MoneyPenny")+ " " + QString(VERSION));
  if(USE_LIVE_CD_FEATURES || DEBUG_HARDWARE){
    buttonDialIn->show();
    spacerDialIn->changeSize(0,10, QSizePolicy::Minimum,QSizePolicy::Minimum);
  }  
  else{
    buttonDialIn->hide(); 
    spacerDialIn->changeSize(0,0, QSizePolicy::Minimum,QSizePolicy::Fixed);
  } 
  _Crypt = new HoBIT_Crypt();
  
  qDebug("Crypt constructed.");
  
  createConfigDir();
 
  textEditHelp->setTextFormat(Qt::RichText);

  _CurrentWizardPage = 0; //Welcome page of wizard
  _AddPrinterProcess = NULL;
  _hobit = NULL;
  _exitCode = EXITCODE_SUCCESS; // normal shutdown - user will be asked
  _AutoShutdown = true;
  _SaveKnoppixConfig = false;
   
   //set all child widgets with NULL
  _guitools = NULL;
  _GuiShowData = NULL;
  _guiwelcome = NULL;
  //_guibalances = NULL;
  _guiaccountinformation = NULL;
  _guitransactions = NULL;
  _guitransactionprinter = NULL;
  _guigettransactions = NULL;
  _guiiniletter = NULL;
  _guisingletransfer = NULL;
  _guidatedsingletransfer = NULL;
  _guidatedtransfers = NULL;
  _guistandingorders = NULL;
  _guistartstandingorder = NULL;
  _guihelpwindow = NULL;
  _guiknownbanks = NULL;
  _guilogin = NULL;
  _guiNullWidget = NULL;
  _guihbciwizard = NULL;
  _guichangeaccountname = NULL;
  _guijobs = NULL;
  _guiaddaccount = NULL;
  _guihardwareconfig = NULL;
  _guidialin= NULL;
  _guilog = NULL;
  _guitransfertemplates = NULL;
  _guitransactioninfo = NULL;
  _guiburn = NULL;			// delete after CeBIT
  _guiSplash = NULL;
  _guiFindMedium = NULL;
  _guiLoginPassword = NULL;
  _guisqlquery = NULL;
  _guisqlqueryresult = NULL;
  _guiprinterautosetup = NULL;
  _guiprintermanualsetup = NULL;
  _guiprinterdrivertoxml = NULL;

   //Initialize splash and findMedium and LoginPassword only. They are needed immediately.
   //All other child widgets are initialized by buildMainApplication while the user looks at the splash.	
   _guiSplash = new GuiSplash();
   qDebug("guiSplash constructed.");
   _guiFindMedium = new GuiFindMedium(_guiSplash);
   qDebug("guiFindMedium constructed.");
   _guiLoginPassword= new GuiLoginPassword(_guiSplash, _Crypt);
   qDebug("guiLoginPassword constructed.");
   _WidgetCallStack = new QValueStack<int>();
   _WidgetCallStack->push(WELCOME_WIDGET);
}


/**Create main configuration directory
*/
void GuiMain::createConfigDir(){
  QString homeDir = getenv("HOME"); //this is on the ramdisk, when running from CD
 
  _ConfigDataDir = homeDir + "/" + MONEYPENNY_MAIN_DIR;
  QDir mpHomeDir(_ConfigDataDir);
  
  if(mpHomeDir.exists()){  //cleanup required, dir exists because of unclean shutdown
   qDebug("Old config data dir found. Removing: %s", _ConfigDataDir.latin1());
   QString rm = "rm -r " + _ConfigDataDir;
   system(rm.latin1());
  }
  
  //now create a new empty dir    
  if(!mpHomeDir.mkdir(_ConfigDataDir)){
     QMessageBox::critical(NULL, QObject::tr("Error"), 
                           QObject::tr("Could not create main configuration data directory: ") + _ConfigDataDir + QObject::tr("Aborting"));
     qDebug("Could not create main configuration data directory: %s", _ConfigDataDir.latin1());
     exit (1);
   }
  
  qDebug("GuiMain::GuiMain: Using _ConfigDataDir: %s", _ConfigDataDir.latin1());
}



/** Builds the widgets for the application and creates the connections. 
  *  Name properties of widgets are used as help file names to control the contents
  *  of the small help window at the lower left. See slotRaiseWidget
  */  
void GuiMain::buildMainApplication(){
   qDebug("Start build application");
  _connector = new HoBIT_Connector(this);
  _hobit = new HoBIT( false,
                      this,
                      MONEYPENNY_OPENHBCI_FILE,
                      MONEYPENNY_AQMONEY_FILE,
                      MONEYPENNY_TEMPLATE_FILE,
                      MONEYPENNY_HBCIDATA_DIR,
                      _ConfigDataDir,
                      APPLICATION_NAME,
                      _connector);

  // create object for internal database management		      
  _sql = new HoBIT_SQL();
                      
  connect(_hobit, SIGNAL(logTextToShow(QString)), this, SLOT(slotAddLogText(QString)));
  connect(_hobit->getAccountJobFactory(), SIGNAL(logTextToShow(QString)), this, SLOT(slotAddLogText(QString)));
  connect(_hobit->getAdminJobFactory(), SIGNAL(logTextToShow(QString)), this, SLOT(slotAddLogText(QString)));
  _guilog = new GuiLog(widgetStack, "LogWidget");
  widgetStack->addWidget(_guilog, LOG_WIDGET);
  connect(_guilog, SIGNAL(closeRequest()), this, SLOT(slotShowLastVisibleWidget()));
  
 // _guiNullWidget = new QWidget(widgetStack, "nullwidget");
 // widgetStack->addWidget(_guiNullWidget, NULL_WIDGET);

  _guiiniletter = new GuiIniLetter(widgetStack, this, "IniLetter", _hobit);
  widgetStack->addWidget(_guiiniletter, INILETTER_WIDGET);
  connect(_guiiniletter, SIGNAL(terminated()), this, SLOT(slotAccountLogin()));
  connect(_guiiniletter, SIGNAL(addPrinterRequest()), this, SLOT(slotHardwareConfig()));

  _guitools = new GuiTools(widgetStack, "ToolBox", _hobit);
  connect(_guitools, SIGNAL(testInternet(const QString& , int, ConnectionState * )), 
           this, SLOT(slotTestInternet(const QString& , int , ConnectionState * )));
  connect(_guitools, SIGNAL(helpRequest(const QString& )), this, SLOT(slotHelp(const QString& )));
  connect(_guitools, SIGNAL(importRDH2Key(const QString & )), this, SLOT(slotImportRDH2Key(const QString & )));
  connect(_guitools, SIGNAL(sqlquery()), this, SLOT(slotSqlQuery()));
  connect(_guitools, SIGNAL(deleteLogFiles()), this, SLOT(slotDeleteLogFiles()));
  connect(_guitools, SIGNAL(setShutdownMode(bool)), this, SLOT(slotSetShutdownMode(bool)));
  connect(_guitools, SIGNAL(timeOutChanged(int)), this, SLOT(slotSetTimeOut(int)));
  connect(_guitools, SIGNAL(back()), this, SLOT(slotShowLastVisibleWidget()));
  connect(_guitools, SIGNAL(saveKnoppixConfiguration()), this, SLOT(slotStoreOsConfig()));
  connect(_guitools, SIGNAL(buildPrinterDriverDatabase()), this, SLOT(slotBuildPrinterDriverDatabase()));
  connect(_guitools, SIGNAL(refreshRequest()), this, SLOT(slotRefresh()));
  connect(_guitools, SIGNAL(raiseWidget(int)), this, SLOT(slotRaiseWidget(int)));
  connect(_guitools, SIGNAL(send()), this, SLOT(slotSend()));
  connect(_guitools, SIGNAL(showIniLetter()), this, SLOT(slotShowIniLetter()));
  widgetStack->addWidget(_guitools, TOOLBOX_WIDGET);
 
  _guiprinterdrivertoxml= new GuiPrinterDriverToXml(widgetStack, "PrinterDriverToXml");
  connect(_guiprinterdrivertoxml, SIGNAL(back()), this, SLOT(slotShowLastVisibleWidget()));
  widgetStack->addWidget(_guiprinterdrivertoxml, PRINTER_DRIVER_TO_XML_WIDGET);
  
  _GuiShowData = new GuiShowData(widgetStack, "ShowData", _hobit);
  connect(_GuiShowData, SIGNAL(helpRequest(const QString& )), this, SLOT(slotHelp(const QString& )));
  connect(_GuiShowData, SIGNAL(back()), this, SLOT(slotShowLastVisibleWidget()));
  widgetStack->addWidget(_GuiShowData, SHOWDATA_WIDGET);

  
  _guiwelcome = new GuiWelcome(widgetStack, "Welcome");
  connect(_guiwelcome, SIGNAL(back()), this, SLOT(slotAccountLogin()));
  widgetStack->addWidget(_guiwelcome, WELCOME_WIDGET);

  qApp->processEvents(); //keep gui alive
 
  _guiaccountinformation = new GuiAccountInformation(widgetStack, this, "AccountInformation", _hobit);
  widgetStack->addWidget(_guiaccountinformation, ACCOUNT_INFORMATION_WIDGET);
  connect(_guiaccountinformation, SIGNAL(deleteAccount(QString)), this, SLOT(slotDeleteAccount(QString)));
  
  _guichangeaccountname = new GuiChangeAccountName(widgetStack, this, "ChangeAccountName", _hobit);
  widgetStack->addWidget(_guichangeaccountname, CHANGE_ACCOUNT_NAME_WIDGET);

  _guitransactions = new GuiTransactions(widgetStack, this,  "Transactions", _hobit);
  widgetStack->addWidget(_guitransactions, TRANSACTIONS_WIDGET);
  connect(_guitransactions, SIGNAL(printTransactions()), this, SLOT(slotPrintTransactions()));
  connect(_guitransactions, SIGNAL(getTransactions()), this, SLOT(slotGetTransactions()));
  connect(_guitransactions, SIGNAL( showDetails(Pointer <Transaction>)), this, SLOT(slotShowTransactionInfo(Pointer <Transaction>)));
  connect(_guitransactions, SIGNAL(exportSelected(QTable*)), this, SLOT(slotExport(QTable*)));
  connect(_guitransactions, SIGNAL(refreshRequest()), this, SLOT(slotRefresh()));
  connect(_guitransactions, SIGNAL(helpRequest(const QString&)), this, SLOT(slotHelp(const QString&)));
  connect(_guitransactions, SIGNAL(raiseWidget(int)), this, SLOT(slotRaiseWidget(int)));

  _guitransactioninfo = new GuiTransactionInfo(widgetStack, this,"TransactionInfo", _hobit);
  widgetStack->addWidget(_guitransactioninfo, TRANSACTION_INFORMATION_WIDGET);
  connect(_guitransactioninfo, SIGNAL(goBack()), this, SLOT(slotShowLastVisibleWidget()));
 
  _guigettransactions = new GuiGetTransactions(widgetStack, this, "GetTransactions", _hobit);
  widgetStack->addWidget(_guigettransactions, GET_TRANSACTIONS_WIDGET);
  connect(_guigettransactions, SIGNAL(raiseWidget(int)), this, SLOT(slotRaiseWidget(int)));
  connect(_guigettransactions, SIGNAL(sendNow()), this, SLOT(slotSendNow()));
  qDebug("GuiGetTransActions constructed");

  _guitransactionprinter = new GuiTransactionPrinter(widgetStack, "TransactionPrinter");
  widgetStack->addWidget(_guitransactionprinter, TRANSACTIONS_PRINTER_WIDGET);
  connect(_guitransactionprinter, SIGNAL(helpRequest(const QString& )), this, SLOT(slotHelp(const QString& )));
  connect(_guitransactionprinter, SIGNAL(cancelled()), this, SLOT(slotShowTransactions()));

  _guisingletransfer = new GuiSingleTransfer(widgetStack, "SingleTransfer", _hobit);
  widgetStack->addWidget(_guisingletransfer, SINGLE_TRANSFER_WIDGET);
  connect(_guisingletransfer, SIGNAL(cancelled()), this, SLOT(slotShowLastVisibleWidget()));
  connect(_guisingletransfer, SIGNAL(raiseWidget(int)), this, SLOT(slotRaiseWidget(int)));
  connect(_guisingletransfer, SIGNAL(sendNow()), this, SLOT(slotSendNow()));  
  connect(_guisingletransfer, SIGNAL(refreshRequest()), this, SLOT(slotRefresh()));

  _guidatedsingletransfer = new GuiDatedSingleTransfer(widgetStack, "DatedSingleTransfer", _hobit);
  widgetStack->addWidget(_guidatedsingletransfer, DATED_SINGLE_TRANSFER_WIDGET);
  connect(_guidatedsingletransfer, SIGNAL(cancelled()), this, SLOT(slotDatedTransfer()));
  connect(_guidatedsingletransfer, SIGNAL(raiseWidget(int)), this, SLOT(slotRaiseWidget(int)));
  connect(_guidatedsingletransfer, SIGNAL(sendNow()), this, SLOT(slotSendNow()));  
  connect(_guidatedsingletransfer, SIGNAL(refreshRequest()), this, SLOT(slotRefresh()));
  
  _guidatedtransfers = new GuiDatedTransfers(widgetStack, this,  "DatedTransfers", _hobit);
  widgetStack->addWidget(_guidatedtransfers, DATED_TRANSFERS_WIDGET);
  connect(_guidatedtransfers, SIGNAL(raiseWidget(int)), this, SLOT(slotRaiseWidget(int)));
  connect(_guidatedtransfers, SIGNAL(showInformation(Pointer<DatedTransfer>)), this,
     SLOT(slotShowDatedTransferInfo(Pointer<DatedTransfer>)));
  
  _guistandingorders = new GuiStandingOrders(this, widgetStack, "StandingOrders", _hobit);
  widgetStack->addWidget(_guistandingorders, STANDING_ORDERS_WIDGET);
  connect(_guistandingorders, SIGNAL(showInformation(Pointer<StandingOrder>)), this,
    SLOT(slotShowStandingOrderInfo(Pointer<StandingOrder>)));
  
  _guistartstandingorder = new GuiStartStandingOrder(widgetStack, "StartStandingOrder", _hobit);
  widgetStack->addWidget(_guistartstandingorder, START_STANDING_ORDER_WIDGET);
  connect(_guistartstandingorder, SIGNAL(cancelled()), this, SLOT(slotStandingOrder()));
  connect(_guistartstandingorder, SIGNAL(raiseWidget(int)), this, SLOT(slotRaiseWidget(int)));
  connect(_guistartstandingorder, SIGNAL(sendNow()), this, SLOT(slotSendNow()));  
  connect(_guistartstandingorder, SIGNAL(refreshRequest()), this, SLOT(slotRefresh()));
  connect(_guistartstandingorder, SIGNAL(raiseWidget(int)), this, SLOT(slotRaiseWidget(int)));

  qApp->processEvents(); //keep gui alive
 
  _guitransfertemplates = new GuiTransferTemplates(widgetStack, "TransferTemplates", _hobit);
  widgetStack->addWidget(_guitransfertemplates, TRANSFER_TEMPLATES_WIDGET);
  connect(_guitransfertemplates, SIGNAL(back()), this, SLOT(slotShowLastVisibleWidget()));
  connect(_guitransfertemplates, SIGNAL(raiseWidget(int)), this, SLOT(slotRaiseWidget(int)));
  connect(_guitransfertemplates, SIGNAL(raiseTemplate(int, QString, QString, QString, QString, QStringList)), 
    this, SLOT(slotRaiseTemplate(int, QString, QString, QString, QString, QStringList)));
  connect(_guisingletransfer, SIGNAL(templateRequest(int)),_guitransfertemplates , SLOT(slotSetTypeRadioButton(int)));
  connect(_guidatedsingletransfer, SIGNAL(templateRequest(int)),_guitransfertemplates , SLOT(slotSetTypeRadioButton(int)));
  connect(_guistartstandingorder, SIGNAL(templateRequest(int)),_guitransfertemplates , SLOT(slotSetTypeRadioButton(int)));
    
  _guihardwareconfig = new GuiHardwareConfig(widgetStack, "HardwareConfig");
  widgetStack->addWidget(_guihardwareconfig, HARDWARE_WIDGET);
  connect(_guihardwareconfig, SIGNAL(finished()), this, SLOT(slotShowLastVisibleWidget()));
  connect(_guihardwareconfig, SIGNAL(raiseWidget(int)), this, SLOT(slotRaiseWidget(int)));
  qDebug("GuiHardwareConfig constructed");
  
  _guidialin = new GuiDialIn(widgetStack, "DialIn");
  widgetStack->addWidget(_guidialin, DIALIN_WIDGET);
  connect(_guidialin, SIGNAL(finished()), this, SLOT(slotShowLastVisibleWidget()));
  qDebug("GuiDialIn constructed");

  _guiknownbanks = new GuiKnownBanks(widgetStack, "KnownBanks", _hobit);
  widgetStack->addWidget(_guiknownbanks, KNOWNBANKS_WIDGET);
  qDebug("GuiKnownBanks constructed");

  _guijobs = new GuiJobs(widgetStack, this, "Jobs", _hobit);
  widgetStack->addWidget(_guijobs, JOBS_WIDGET);
  qDebug("GuiJobs constructed");

  _guiaddaccount = new GuiAddAccount(widgetStack, this, "AddAccount", _hobit);
  widgetStack->addWidget(_guiaddaccount, ADD_ACCOUNT_WIDGET);
  qDebug("GuiAddAccount constructed");

  _guihbciwizard = new GuiHbciWizard(this, "HbciWizard", _hobit);
  widgetStack->addWidget(_guihbciwizard, HBCI_WIZARD);
  connect(_guihbciwizard, SIGNAL(helpRequest(const QString& )), this, SLOT(slotHelp(const QString& )));
  connect(_guihbciwizard, SIGNAL(setupSuccessful(HBCI_MODE)), this, SLOT(slotSetupSuccessful(HBCI_MODE)));
  connect(_guihbciwizard, SIGNAL(setupInternet()), this, SLOT(slotHardwareConfig()));
  connect(_guihbciwizard, SIGNAL(showLog(int)), this, SLOT(slotRaiseWidget(int)));
  connect(_guihbciwizard, SIGNAL(testInternet(const QString& , int, ConnectionState * )), 
           this, SLOT(slotTestInternet(const QString& , int , ConnectionState * )));
  connect(_guihbciwizard, SIGNAL(renewHoBIT()), this, SLOT(renewHoBIT()));
  connect(_guihbciwizard, SIGNAL(addAccount()), this, SLOT(slotAddAccount()));
  connect(_guihbciwizard, SIGNAL(dialIn()), this, SLOT(slotDialIn()));
  connect(_guihbciwizard, SIGNAL(aborted()), this, SLOT(slotHbciSetupAborted()));
  connect(_guihbciwizard, SIGNAL(wizardPageChanged(int)), this, SLOT(slotWizardPageChanged (int)));
 
  connect(_guiaddaccount, SIGNAL(accountAdded()), _guihbciwizard, SLOT(findAccounts()));
  qDebug("GuiHbciWizard constructed");
 
  qDebug("Running from: %s", QDir::currentDirPath().latin1());
  QStringList helpDirs;
  helpDirs << "." << "help";
  _guihelpwindow = new GuiHelpWindow(widgetStack, "HelpWindow", MP_DIR() + MONEYPENNY_HELP_DIR + "/index.html", helpDirs);
  widgetStack->addWidget(_guihelpwindow, HELP_WIDGET);
  connect(_guihelpwindow, SIGNAL(cancelled()), this, SLOT(slotShowLastVisibleWidget()));

  qDebug("Creating guiLogin");

  _guilogin = new GuiLogin(widgetStack, this, "Login", _hobit);
  widgetStack->addWidget(_guilogin, LOGIN_WIDGET);
  
 _guisqlquery = new GuiSqlQuery(widgetStack, "SqlQuery"); 
  widgetStack->addWidget(_guisqlquery, SQL_QUERY_WIDGET);
  connect(_guisqlquery, SIGNAL(sqlQuery(QString)), this, SLOT(slotExecuteSqlQuery(QString)));

 _guisqlqueryresult = new GuiSqlQueryResult(widgetStack, "SqlQueryResult");
  widgetStack->addWidget(_guisqlqueryresult, SQL_RESULT_WIDGET);
  connect(_guisqlqueryresult, SIGNAL(goback()), this, SLOT(slotShowLastVisibleWidget()));
  connect(_guisqlqueryresult, SIGNAL(exportdata(QTable*)), this, SLOT(slotExport(QTable*)));
  
  _guidataexport = new GuiDataExport(widgetStack, "Export");
  widgetStack->addWidget(_guidataexport, EXPORT_WIDGET);
  connect (_guidataexport, SIGNAL(back()), this, SLOT(slotShowLastVisibleWidget()));
  connect (_guidataexport, SIGNAL(exportSQL(QString)), this, SLOT(slotMakeSQL(QString)));
  
  _guiprinterautosetup = new GuiPrinterAutoSetup(widgetStack, "PrinterAutoSetup");
  widgetStack->addWidget(_guiprinterautosetup, PRINTER_AUTOSETUP_WIDGET);
  connect(_guiprinterautosetup, SIGNAL(back()), this, SLOT(slotShowLastVisibleWidget()));
  connect(_guiprinterautosetup, SIGNAL(helpRequest(const QString& )), this, SLOT(slotHelp(const QString&)));
  connect(_guiprinterautosetup, SIGNAL(showSmallHelp(const QString& )), this, SLOT(slotShowSmallHelp(const QString&)));
  connect(_guiprinterautosetup, SIGNAL(raiseWidget(int)), this, SLOT(slotRaiseWidget(int)));

  _guiprintermanualsetup = new GuiPrinterManualSetup(widgetStack, "PrinterManualSetup");
  widgetStack->addWidget(_guiprintermanualsetup, PRINTER_MANUALSETUP_WIDGET);
  connect(_guiprintermanualsetup, SIGNAL(showSmallHelp(const QString& )), this, SLOT(slotShowSmallHelp(const QString&)));
  connect(_guiprintermanualsetup, SIGNAL(back()), this, SLOT(slotShowLastVisibleWidget()));
  connect(_guiprintermanualsetup, SIGNAL(helpRequest(const QString& )), this, SLOT(slotHelp(const QString&)));

  // delete after cebit
  if (USE_CEBIT_FEATURES){
  _guiburn = new GuiBurn(widgetStack, "Burn");
   widgetStack->addWidget(_guiburn, BURN_WIDGET);
  }

  slotRaiseWidget(LOGIN_WIDGET);

  createMenu();

  //start Timer
  _timer = new QTimer( this );
  connect( _timer, SIGNAL(timeout()), this, SLOT(slotTimerDone()) );
  _timer->start( 1000, FALSE ); // 10 seconds multishot timer

  //show tcp-status
  textLabelConnection->setText(tr("Initialization"));
  _SmallHelp = "smallMain.html";
}

/**
*destructor of GuiMain
*/
GuiMain::~GuiMain()
{
  delete _guiSplash; //_guiSplash is a top level widget. Needs to be deleted.
  delete _Crypt;
  if(_hobit)
   delete _hobit;
  if(_WidgetCallStack)
    delete _WidgetCallStack; 
  qDebug("Destructor GuiMain finished");
}

/**
*Raises the HBCI-Widget on the WidgetStack and resets the banking engine.
* This allows to setup a new bank from scratch. 
*/
void GuiMain::slotHbciSetup()
{
  if(QMessageBox::question(this,
      tr("Confirmation"),
      tr("Do you really want to renew the HBCI-setup for the following bank?")+ "\n"+
      tr("Selected bank:") +" "+_hobit->getBankNameForId(_hobit->getBankId()) + " "+
      tr("Bank code:") +" " +_hobit->getBankId(),
      tr("No"), tr("Yes"))!=0)
  {    
      slotRaiseWidget(HBCI_WIZARD);
      _hobit->removeUncryptedEnvironment(); //remove all data
      createConfigDir();    //and start from scratch with an empty dir
      renewHoBIT();
      _hobit->loadEnvironment();
      _guihbciwizard->evaluateStoredData();
      textLabelStatus->setText(tr("HBCI-Configuration"));
      slotRefresh();
  }
  else return;
}

/**
*Raises the toolbox widget 
*/
void GuiMain::slotToolbox()
{
  slotRaiseWidget(TOOLBOX_WIDGET);
  textLabelStatus->setText(tr("Toolbox"));
 // slotShowSmallHelp("smallToolbox.html");
  slotRefresh();
}

/**
*Raises the Widget, which shows quick information about all hbci accounts 
*/
void GuiMain::slotAccounts()
{
  slotRaiseWidget(ACCOUNT_INFORMATION_WIDGET);
  textLabelStatus->setText(tr("Account Status"));
  //   _guibalances->updateList();
  _guiaccountinformation->refresh();
 slotRefresh();
}

/**
* asks for execution of the job list and sends if requested.
*/
void GuiMain::slotSend()
{
   switch (askForSend() ){
     case 0: break;
     case 1: slotSendNow(); break;
     case 2: {
               slotGetTransactions();
               slotSendNow(); 
               break;
             } 
     default: break;       
    } 
   
}
/**
*starts execution of the job list 
*/
void GuiMain::slotSendNow()
{
   qDebug("GuiMain::slotSendNow called.");
   slotRaiseWidget(LOG_WIDGET);
   update();
   if(_hobit->areJobQueuesEmpty()){
     QMessageBox::information(this, tr("Information"), tr("Job queue is empty. No jobs to be sent."));
     return;
   }

   if (_hobit->getHbciMode()== HBCI_MODE_DDV){
    if(!_hobit->isCardReaderOk()){
      QMessageBox::critical(this, tr("Error"), tr("Could not contact security card reader. Is DDV2 plugin installed? Is reader plugged in?"));
      return;
    }
   }

   QString logText= tr("Executing Job Queue");
   QCursor oldCursor = cursor();
   setCursor(QCursor(Qt::WaitCursor));
   _guilog->setLogText(logText);
   if (_hobit->executeJobQueue()== HobitErrorNotConnected)
      slotRaiseWidget(WELCOME_WIDGET);
   else {
     _guilog->showResults();
    slotRefresh();
   }
   setCursor(oldCursor);
}

/**
*Select the accounts for which the transactionlists are to be updated 
*/
void GuiMain::slotGetTransactions()
{
  slotRaiseWidget(GET_TRANSACTIONS_WIDGET);
}


/**
*shows a dialog with the turnovers 
*/
void GuiMain::slotShowTransactions()
{
    slotRaiseWidget(TRANSACTIONS_WIDGET);
    textLabelStatus->setText(tr("Turnovers"));
   // slotShowSmallHelp("smallTransactions.html");
   slotRefresh();
}

/**
*creates a single transfer 
*/
void GuiMain::slotSingleTransfer()
{
  slotRaiseWidget(SINGLE_TRANSFER_WIDGET);
  textLabelStatus->setText(tr("Single Transfer"));
//  slotShowSmallHelp("smallSingleTransfer.html");
  slotRefresh();
}

/**
*creates a dated transfer
*/
void GuiMain::slotDatedTransfer()
{
   slotRaiseWidget(DATED_TRANSFERS_WIDGET);
   textLabelStatus->setText(tr("Dated Transfer"));
  // slotShowSmallHelp("smallDatedTransfer.html");
   slotRefresh();
}

/**
*calls a transfer template. This is a user defined mask for frequently used transfers. 
*/
void GuiMain::slotTransferTemplate()
{
  slotRaiseWidget(TRANSFER_TEMPLATES_WIDGET);
  textLabelStatus->setText(tr("Transaction Template"));
 // slotShowSmallHelp("smallTransferTemplate.html");
 }

/**
*creates a standing order
*/
void GuiMain::slotStandingOrder()
{
  slotRaiseWidget(STANDING_ORDERS_WIDGET);
  textLabelStatus->setText(tr("Standing Order"));
//  slotShowSmallHelp("smallStandingOrder.html");
 slotRefresh();
}


/**
*Update slots
*/
void GuiMain::slotUpdate()
{
  //_hobit->createJobGetAccounts();
  _hobit->createJobGetBalance();
  _hobit->createJobGetTransactions(Date(), Date(), "*");
  _hobit->createJobGetDatedTransfers(Date(), Date(), "*");
  _hobit->createJobGetStandingOrders("*");
  slotSendNow();
}


/**
*Raise widget for undone jobs 
*/
void GuiMain::slotJobs()
{
  slotRaiseWidget(JOBS_WIDGET);
  textLabelStatus->setText(tr("Job-Queue"));
 // slotShowSmallHelp("smallJoblist.html");
 slotRefresh();
}

/**
*Raise hardware widget in CONNECT mode 
*/
void GuiMain::slotDialIn()
{
  slotRaiseWidget(DIALIN_WIDGET);
  //slotShowSmallHelp("smalldialin.html");
}

/**
*refresh informations, lists and widgets
*/
void GuiMain::slotRefresh()
{
  _guiaccountinformation->refresh();
  _guilogin->refresh();
  _guitools->refresh();
  _guitools->setTimeOut(_connector->getTimeOut());
  _GuiShowData->refresh();
  _guistandingorders->refresh();
  _guidatedtransfers->refresh();
  _guitransactions->refresh();
  _guitransactionprinter->refresh();
  _guitransfertemplates->refresh();
  _guitransactioninfo->refresh();
  _guisingletransfer->refresh();
  _guidatedsingletransfer->refresh();
  _guistartstandingorder->refresh();
  _guijobs->refresh();
  _guijobs->repaint();
}

/**
*Building Menu by adding items and setting labels, bottom up
*/
void GuiMain::createMenu()
{
    if (!USE_LIVE_CD_FEATURES && USE_CEBIT_FEATURES){
    _cebitItem = new QListViewItem(listViewMenu, tr("CeBIT 2005"));
    _burnItem = new QListViewItem(_cebitItem, tr("Burn Live-Cd"));
  }
  else{
   _cebitItem = NULL;
   _burnItem = NULL;
  }
  
  listViewMenu->setSorting(-1); //switch off sorting
  _exitItem = new QListViewItem(listViewMenu, tr("Exit"));
  
  _helpItem = new QListViewItem(listViewMenu, tr("Help"));
  _updateItem = new QListViewItem(listViewMenu, tr("Program Update"));
  _knownBanksItem = new QListViewItem(listViewMenu, tr("German Banks"));
  
  _configItem = new QListViewItem(listViewMenu, tr("Setup"));
  _ChangeLoginPasswordItem = new QListViewItem(_configItem, tr("Change Login-Password"));
  _ChangeHbciPasswordItem = new QListViewItem(_configItem, tr("Change HBCI-Password"));
  _CreateBackupMediumItem = new QListViewItem(_configItem, tr("Create Backup Medium"));
  _ToolboxItem = new QListViewItem(_configItem, tr("Toolbox"));
  _ShowDataItem = new QListViewItem(_configItem, tr("Show User Data"));
  _updateAccountList= new QListViewItem(_configItem, tr("Update Account List")); 
  _AddBankItem= new QListViewItem(_configItem, tr("Add Bank")); 
  _hbciItem = new QListViewItem(_configItem, tr("HBCI-Setup"));
  if(USE_LIVE_CD_FEATURES || DEBUG_HARDWARE)
   _hardwareItem = new QListViewItem(_configItem, tr("Hardware"));
  else _hardwareItem = NULL;
  
  if(USE_LIVE_CD_FEATURES)
   _connectItem = new QListViewItem(listViewMenu, tr("Internet Dial In"));
  else _connectItem = NULL;
     
  _jobItem = new QListViewItem(listViewMenu, tr("Jobs"));
  _sendNowItem = new QListViewItem(_jobItem, tr("Execute Job Queue"));
  _jobQueueItem = new QListViewItem(_jobItem, tr("Job-Queue"));
    
  _paymentItem = new QListViewItem(listViewMenu, tr("Payment"));
  _standingOrderItem = new QListViewItem(_paymentItem, tr("Standing Orders"));
  _datedTransferItem = new QListViewItem(_paymentItem, tr("Dated Transfer"));
  _singleTransferItem = new QListViewItem(_paymentItem, tr("Single Transfer"));
  _transferTemplateItem = new QListViewItem(_paymentItem, tr("Transfer Template"));

//  _turnOverItem = new QListViewItem(listViewMenu, tr("Turnovers"));
  _accountItem= new QListViewItem(listViewMenu, tr("Account Status"));
  _exportTransactionsItem = new QListViewItem(_accountItem, tr("Export Turnover Data"));
  _printTransactionsItem = new QListViewItem(_accountItem, tr("Print Turnovers"));
  _showTransactionsItem = new QListViewItem(_accountItem, tr("Show Turnovers"));
  _getTransactionsItem = new QListViewItem(_accountItem, tr("Get Turnovers"));

  _accountBalanceItem= new QListViewItem(_accountItem, tr("Show Account Status"));

  _selectAccountItem= new QListViewItem(listViewMenu, tr("Select an Account"));

  _changeBankItem= new QListViewItem(listViewMenu, tr("Change Bank")); 

}

/**
*execute slot after pressing an item of the menu
*/
void GuiMain::slotSelectItemFromMenu( QListViewItem * item )
{ 
  if(item == NULL) 
    return;
  else if (_accountItem && _accountItem->isSelected()) slotShowWelcome("smallAccounts.html"); 
  else if (_changeBankItem && _changeBankItem->isSelected()) slotChangeBank();
  else if (_selectAccountItem && _selectAccountItem->isSelected()) slotAccountLogin();
  else if (_accountBalanceItem && _accountBalanceItem->isSelected()) slotAccounts();
  else if (_updateAccountList && _updateAccountList->isSelected()) slotUpdateAccountList();
  
 // else if (_turnOverItem && _turnOverItem->isSelected()) slotShowWelcome("smallTurnovers.html"); 
  else if (_getTransactionsItem && _getTransactionsItem->isSelected()) slotGetTransactions();
  else if (_showTransactionsItem && _showTransactionsItem->isSelected()) slotShowTransactions();
  else if (_printTransactionsItem && _printTransactionsItem->isSelected()) slotPrintTransactions();
  else if (_exportTransactionsItem && _exportTransactionsItem->isSelected()) slotShowTransactions();
  
  else if (_paymentItem && _paymentItem->isSelected())  slotShowWelcome("smallPayment.html"); 
  else if (_singleTransferItem && _singleTransferItem->isSelected()) slotSingleTransfer();
  else if (_datedTransferItem && _datedTransferItem->isSelected()) slotDatedTransfer();
  else if (_transferTemplateItem && _transferTemplateItem->isSelected()) slotTransferTemplate();
  else if (_standingOrderItem && _standingOrderItem->isSelected()) slotStandingOrder();
   
  else if (_configItem && _configItem->isSelected())  slotShowWelcome("smallConfig.html"); 
  else if (_hbciItem && _hbciItem->isSelected()) slotHbciSetup();
  else if (_AddBankItem && _AddBankItem->isSelected()) slotAddBank();
  else if (_ChangeLoginPasswordItem && _ChangeLoginPasswordItem->isSelected()) slotChangeLoginPassword();
  else if (_ChangeHbciPasswordItem && _ChangeHbciPasswordItem->isSelected()) slotChangeHbciPassword();
  else if (_CreateBackupMediumItem && _CreateBackupMediumItem->isSelected()) slotCreateBackupMedium();
  else if (_hardwareItem && _hardwareItem->isSelected()) slotHardwareConfig();
  else if (_ToolboxItem && _ToolboxItem->isSelected()) slotToolbox();
  else if (_ShowDataItem && _ShowDataItem->isSelected()) slotShowData();
  
  else if (_connectItem && _connectItem->isSelected())  slotDialIn(); 
  else if (_jobItem && _jobItem->isSelected())  slotShowWelcome("smallJobs.html"); 
  else if (_jobQueueItem && _jobQueueItem->isSelected()) slotJobs();
  else if (_sendNowItem && _sendNowItem->isSelected()) slotSendNow();
  
  else if (_knownBanksItem && _knownBanksItem->isSelected()) slotListKnownBanks();
  
  else if (_helpItem && _helpItem->isSelected()) slotHelp("index.html");

  else if (_updateItem && _updateItem->isSelected()) slotUpdateProgram();
  
  else if (_exitItem && _exitItem->isSelected()) slotClose();
      
  else if (_burnItem && _burnItem->isSelected()) slotRaiseWidget(BURN_WIDGET);
  
  //toggle open state
  item->setOpen(! item->isOpen());
}


/**
*referring to statusbar:
*- get internet connectionState from connector
*- update date and time
*/
void GuiMain::slotTimerDone()
{
  changeBalanceStatus();

  ConnectionState connectionState = _connector->connectionState();
  switch (connectionState){
    case ConnectionUntested  : {
                                 textLabelConnection->setPaletteBackgroundColor(white);
                                 textLabelConnection->setText(" " +tr("Internet: Untested") + " ");
                                 break;
                               } 
    case ConnectionConnecting: {
                                 textLabelConnection->setPaletteBackgroundColor(yellow);
                                 textLabelConnection->setText(" " + tr("Internet: Connecting") + " ");
                                 break;
                               } 
    case ConnectionFailed    : {
                                 textLabelConnection->setPaletteBackgroundColor(red);
                                 textLabelConnection->setText(" " + tr("Internet: Failed") + " ");
                                 break;
                               } 
    case ConnectionConnected : {
                                 textLabelConnection->setPaletteBackgroundColor(green);
                                 textLabelConnection->setText(" " + tr("Internet: Online") + " ");
                                 break;
                               } 
    case ConnectionClosed    : {
                                textLabelConnection->setPaletteBackgroundColor(yellow);
                                textLabelConnection->setText(" " + tr("Internet: Offline") + " ");
                                break;
                               } 
    case ConnectionWasOk     : {
                                 textLabelConnection->setPaletteBackgroundColor(yellow);
                                 textLabelConnection->setText(" " + tr("Internet: Ok") + " ");
                                 break;
                               }

   default                   : {
                                 textLabelConnection->setPaletteBackgroundColor(yellow);
                                 textLabelConnection->setText(" " + tr("Internet: ??") + " ");
                                 break;
                               } 
  }
  QDate currentDate;
  QTime currentTime;
  QString date = currentDate.currentDate().toString();
  QString time = currentTime.currentTime().toString();
  QString datetime = date + " - " + time;
  textLabelDate->setText(datetime);
}

/**
*refresh account-balance
*/
void GuiMain::changeBalanceStatus()
{
  if (_hobit->getCurrentAccount() != "")
  {
    Pointer<Account> acc = _hobit->findAccount(_hobit->getCurrentAccount().latin1());
    if (acc.isValid())
    {
      QString balance, n;
      balance = acc.ref().bookedBalance().value().toReadableString().c_str();
      QColor fgColor;
      if(acc.ref().bookedBalance().value().getValue() > 0)
        fgColor = "black";
      else fgColor = "red"; 
      labelAccount->setText(_hobit->getCurrentAccount());
      labelBalance->setText(balance);
      labelBalance->setPaletteForegroundColor(fgColor);
    }
    else{ 
      labelBalance->setText(tr("Unknown"));
      labelAccount->setText(tr("No valid Account"));
    }
  }
  else {
    labelBalance->setText(tr("Unknown"));
    labelAccount->setText(tr("Not selected"));
  }  
}

/**
*Prints the transactions.
*/
void GuiMain::slotPrintTransactions()
{
  if (!_printTransactionsItem->isSelected()) //synchronize menu
    listViewMenu->setSelected(_printTransactionsItem, true);
  else
  {
    _guitransactionprinter->setAccountName(getCurrentAccount());
    _guitransactionprinter->setTransactionList(_guitransactions->getTransactionList());
    slotRaiseWidget(TRANSACTIONS_PRINTER_WIDGET);
  }
}

/**
*Call slotHelp (QString&)
*/
void GuiMain::slotHelp()
{
 slotHelp("index.html");
}

/**
*Open the help window
*@param helpfile a helping QString
*/
void GuiMain::slotHelp(const QString & helpfile)
{
  _guihelpwindow->setHelpfile (MP_DIR() + MONEYPENNY_HELP_DIR  + "/" +  helpfile);
  slotRaiseWidget(HELP_WIDGET);
}


/**
*set Menubar and insert accountbalance
*/
void GuiMain::doMenu()
{
  bar = new QMenuBar(this);
  bankingMenu = new QPopupMenu(this);
  bankingMenu->insertItem("&Balance",  this, SLOT(slotAccounts()), CTRL+Key_K );
  bar->insertItem("&Banking", bankingMenu);
}

/**
*Raise widget with list of known banks
*/
void GuiMain::slotListKnownBanks()
{
  slotRaiseWidget(KNOWNBANKS_WIDGET);
  textLabelStatus->setText(tr("german banks"));
  //slotShowSmallHelp("smallKnownBanks.html");
}


/**
*set account to be used from moneypenny after login 
*@param ca QString for the current account
*/
void GuiMain::setCurrentAccount(QString &ca)
{
  _hobit->setCurrentAccount(ca);
  slotActivateMenuFunctions();
  slotRefresh();
  slotRaiseWidget(WELCOME_WIDGET);
  textLabelStatus->setText(tr("Account "));
}


/**
*set account to be used from moneypenny after login 
*/
QString GuiMain::getCurrentAccount()
{
  return _hobit->getCurrentAccount();
}


/**
*Raise widget for ini-letter
*/
void GuiMain::slotShowIniLetter()
{
  _guiiniletter->newIniLetter();
  slotRaiseWidget(INILETTER_WIDGET);
}

/**
*Switch Account by raise login-widget
*/
void GuiMain::switchAccount()
{
  slotAccountLogin();
}

/**
*Change Accoutname
*/
void GuiMain::changeAccountName()
{
  slotRaiseWidget(CHANGE_ACCOUNT_NAME_WIDGET);
}

/**
*Show last visible widget
*/
void GuiMain::slotShowLastVisibleWidget()
{
  int widgetId = _WidgetCallStack->pop();
  widgetStack->raiseWidget(widgetStack->widget(widgetId));
}

/**
*Raise widget for add account
*/
void GuiMain::slotAddAccount()
{
  slotRaiseWidget(ADD_ACCOUNT_WIDGET);
}

/** Deletes the account accountid by deleting its account directory.
*/
void GuiMain::slotDeleteAccount(QString accountId){
 _hobit->deleteAccount(accountId);
 renewHoBIT();
 slotRefresh();
}

/**
*Raise widget for hardware-configure-dialog
*/
void GuiMain::slotHardwareConfig()
{
  slotRaiseWidget(HARDWARE_WIDGET);
  textLabelStatus->setText(tr("Hardware Setup"));
// slotShowSmallHelp("smallHardwareSetup.html");
  _SaveKnoppixConfig = true;
  qDebug("GuiMain::slotHardwareConfig: setting SaveKnoppixConfig flag");
}


/**
*Raise Login-Dialog and HBCI-Wizard
*/
void GuiMain::slotAccountLogin()
{ 
  slotDeactivateMenuFunctions();
  list<Pointer<Account> > accounts = _hobit->listAccounts();
  list<Pointer<User> > users = _hobit->listUsers();
  if(users.empty()){ // no user, start from scratch
         _guihbciwizard->showPage(_guihbciwizard->page(WELCOME_PAGE));    
          slotRaiseWidget(HBCI_WIZARD);
  }  
  else if (!users.empty() && accounts.empty()){ // no accounts for existing user
         _guihbciwizard->evaluateStoredData();
         _guihbciwizard->showPage(_guihbciwizard->page(CONNECT_PAGE));
         slotRaiseWidget(HBCI_WIZARD);         
   }  
  else
  {
    _guilogin->refresh();
    slotRaiseWidget(LOGIN_WIDGET);
  }  
}

/**
*Close main window after confirming
*/
void GuiMain::slotClose()
{
 _exitCode = EXITCODE_SUCCESS;
 close();
}

/**
* Change bank by shutting down and restarting with new medium 
*/
void GuiMain::slotChangeBank()
{
 _exitCode = EXITCODE_RUN_BANK_LOOP;
 close();
}

/**
*Raises widget with given id
*Builds the name of the small help file from widget name
*@param id id of the widget which should be raised 
*/
void GuiMain::slotRaiseWidget(int id)
{
  if((id >=0) && widgetStack->visibleWidget())
  {
    int currentId = widgetStack->id(widgetStack->visibleWidget());
    if(currentId != id){ 
       _WidgetCallStack->push(widgetStack->id(widgetStack->visibleWidget())); //save current page id        
       widgetStack->raiseWidget(id);  //finally raise the new widget   
       if(id == HBCI_WIZARD)
         slotWizardPageChanged(_guihbciwizard->indexOf(_guihbciwizard->currentPage()));
       else {
         _SmallHelp = "small" + QString(widgetStack->visibleWidget()->name()) + ".html"; //Use the name as identifier for help file control
         slotShowSmallHelp(_SmallHelp); //now show it
       }
    }
  }
}

/** Updates _CurrentWizardPage and contents of small help widget
*/
void GuiMain::slotWizardPageChanged(int newPage){
    _CurrentWizardPage = newPage; 
    if(widgetStack->visibleWidget() == _guihbciwizard){  //we are on top, show the small help file    
      //build the name of the small help file from widget name
      _SmallHelp = "small" + QString(widgetStack->visibleWidget()->name()); //Use the caption as identifier for help file control
      int page = _CurrentWizardPage;
      if(page < 0)
         page = 0; //do not call help for negative numbers
      _SmallHelp += QString::number(page); //add the page number of the wizard page
      _SmallHelp += ".html"; 
      slotShowSmallHelp(_SmallHelp); //now show it
    }
}

/** Displays helpFile in the small help widget at the lower left.
*/
void GuiMain::slotShowSmallHelp(const QString & helpFile){
  _SmallHelp = helpFile;
  QString fileName = MP_DIR()  + MONEYPENNY_HELP_DIR + "/" + helpFile;
  QFile file( fileName); // Read the text from a file
  if ( file.open( IO_ReadOnly ) )
  {
    QTextStream stream( &file );
    textEditHelp->setText( stream.read() );
    file.close();
  }
  else QMessageBox::warning(this, tr("File Error"), tr("Could not open help file : ") 
                                 + fileName + tr("<br>Please check your installation."));
}


/**
*function for closing
*/
void GuiMain::closeEvent(QCloseEvent * ce)
{
  if (_exitCode == EXITCODE_RESTART_SESSION){//immediate shutdown and restart.
    QMessageBox::information(this,
               tr("Information"),
               tr("MoneyPenny will now terminate the currrent session without saving data.") +"\n"+                
               tr("After this it will start a new session.")); 
               _hobit->removeUncryptedEnvironment();  
               ce->accept();
               QApplication::exit(_exitCode);               
               return;
  } 
   
  if (! _hobit->areJobQueuesEmpty()){
    if (QMessageBox::question( this,
                              tr("MoneyPenny Shutdown"),
                              tr("You have unsent jobs in your job queue. \nDo you really want to close MoneyPenny (Unsent jobs will be lost)?"),
                              tr("&Ok"), tr("&Cancel"),
                              QString::null, 0, 1 )){
	 ce->ignore();
	 return;
   }
  }
  QString sStorage;
  switch (_Storage){
    case STORAGE_FLOPPY  : sStorage = tr("floppy disk in the drive", 
                           "Replacement for sStorage in: please leave your  sStorage  and press Ok.");break;
    case STORAGE_USB     : sStorage = tr("USB stick plugged in", 
                           "Replacement for sStorage in: please leave your  sStorage  and press Ok.");    break;
    case STORAGE_UNKNOWN : sStorage = tr("storage device in place", 
                           "Replacement for sStorage in: please leave your  sStorage  and press Ok."); break;
    default              : sStorage = tr("storage device in place", 
                           "Replacement for sStorage in: please leave your  sStorage  and press Ok."); break;
  }
  bool bSaveData;
  if (_exitCode == EXITCODE_SUCCESS){
     int result = QMessageBox::question
               ( this,
               tr("MoneyPenny Shutdown"),
               tr("If you want to close MoneyPenny, \nplease leave your ") + sStorage + tr(" and press Ok."),
               tr("&Ok"), 
               tr("&Cancel"));
               bSaveData = result == 0; //First (ok) button has been clicked
   } 
  else if (_exitCode == EXITCODE_RUN_BANK_LOOP){
      int result =QMessageBox::question
               ( this,
               tr("MoneyPenny Changing Bank"),
               tr("MoneyPenny will now save all data for your present bank, \nplease leave your ") + sStorage + tr(" and press Ok."),
               tr("&Ok"), 
               tr("&Cancel"));
               bSaveData = result == 0 ;//First (ok) button has been clicked
  } 
  else bSaveData = false;
  if (bSaveData){  
        textLabelStatus->setText(tr("Saving data and configuration"));      
        qApp->processEvents();
        switch (_Storage){
          case STORAGE_FLOPPY  : sStorage = tr("floppy disk", 
                                 "This is the end of the sentence: Could not write to your floppy disk");    break;
          case STORAGE_USB     : sStorage = tr("USB stick", 
                                 "This is the end of the sentence: Could not write to yourUSB stick");       break;
          case STORAGE_UNKNOWN : sStorage = tr("storage device", 
                                 "This is the end of the sentence: Could not write to your storage device"); break;
          default              : sStorage = tr("storage device", 
                                 "This is the end of the sentence: Could not write to your storage device"); break;
        }

        if(_AutoShutdown && (_Storage == STORAGE_FLOPPY)) //do not save logfiles on a floppy, if everything was ok
          slotDeleteLogFiles();
                
        if(_SaveKnoppixConfig)
           slotStoreOsConfig();
                
        if(!_hobit->saveEnvironment()){ //save HBCI configuration to ConfigDataDir. 
          bool closeAnyway= QMessageBox::question( this,
                              tr("Error"),
                              tr(" <b>Saving of data failed.</b>"),
                              tr("&Cancel"), 
                              tr("&Close Anyway"),
                              QString::null, 0, 1 ) == 1;
          if(closeAnyway) 
            ce->accept();
          else ce->ignore();
          return;                      
        }  
        
                       
        //remove ConfigDataDir           
        if(encryptData())  //First compress and encrypt ConfigDataDir.
          _hobit->removeUncryptedEnvironment();
        
        switch (_Storage){
          case STORAGE_FLOPPY  : sStorage = tr("floppy disk", 
                                 "Replacement for sStorage in: Please remove your  sStorage  now and press Ok."); break;
          case STORAGE_USB     : sStorage = tr("USB stick", 
                                 "Replacement for sStorage in: Please remove your  sStorage  now and press Ok."); break;
          case STORAGE_UNKNOWN : sStorage = tr("storage device", 
                                 "Replacement for sStorage in: Please remove your  sStorage  now and press Ok."); break;
          default              : sStorage = tr("storage device", 
                                 "Replacement for sStorage in: Please remove your  sStorage  now and press Ok."); break;
        }
       bool ok = QMessageBox::question
         ( this,
          tr("MoneyPenny Shutdown"),
          tr("Data have been encrypted and saved. \nPlease remove your ") + sStorage 
          + tr(" after the indicator lamp of the drive switches off and press Ok."),               
          tr("&OK")  , tr("&Cancel"))
         == 0 ;

       if(ok){
        textLabelStatus->setText(tr("Terminating session")); 
        if(USE_LIVE_CD_FEATURES && (!_AutoShutdown))
         _exitCode = EXITCODE_NO_AUTOSHUTDOWN;
        QApplication::exit(_exitCode); 
       } else ce->ignore(); // cancel was selected after data saving
  }else ce->ignore(); // cancel was selected after first question   
   
}

/**
*Deactivate Buttons and List
*/
void GuiMain::slotDeactivateMenuFunctions()
{
  buttonAccounts->setEnabled(false);
  buttonTransfer->setEnabled(false);
  buttonTurnover->setEnabled(false);
  buttonUpdate->setEnabled(false);
  buttonSendNow->setEnabled(false);
  //buttonToolbox->setEnabled(false);
 
  _accountItem->setEnabled(false);
  _paymentItem->setEnabled(false);
  _jobItem->setEnabled(false);
  //_configItem->setEnabled(false);
  if(_connectItem)
      _connectItem->setEnabled(false);
  if(_hobit->listAccounts().empty()){
      _selectAccountItem->setEnabled(false);
      _accountBalanceItem->setEnabled(false);
   }
  _accountBalanceItem->setEnabled(false);
  _updateAccountList->setEnabled(false);
  
  //_turnOverItem->setEnabled(false);
  _getTransactionsItem->setEnabled(false);
  _showTransactionsItem->setEnabled(false);
  _printTransactionsItem->setEnabled(false);
  _exportTransactionsItem->setEnabled(false);
  
  _singleTransferItem->setEnabled(false);
  _datedTransferItem->setEnabled(false);
  _transferTemplateItem->setEnabled(false);
  _standingOrderItem->setEnabled(false);
  
  _jobQueueItem->setEnabled(false);
  _sendNowItem->setEnabled(false);
  
  _ChangeLoginPasswordItem->setEnabled(false);
  _ChangeHbciPasswordItem->setEnabled(false);
  _CreateBackupMediumItem->setEnabled(false);
  _ToolboxItem->setEnabled(false);
  _ShowDataItem->setEnabled(false);
  
  
}

/**
*Activate Buttons and List
*/
void GuiMain::slotActivateMenuFunctions()
{
  buttonAccounts->setEnabled(true);
  buttonTransfer->setEnabled(true);
  buttonTurnover->setEnabled(true);
  buttonUpdate->setEnabled(true);
  buttonSendNow->setEnabled(true);
  buttonToolbox->setEnabled(true);
 // listViewMenu->setEnabled(true);
  
  _paymentItem->setEnabled(true);
  _jobItem->setEnabled(true);
  _configItem->setEnabled(true);
  if(_connectItem) _connectItem->setEnabled(true);

  _accountItem->setEnabled(true);
  _updateAccountList->setEnabled(true);
  _selectAccountItem->setEnabled(true);
  _accountBalanceItem->setEnabled(true);
  //_turnOverItem->setEnabled(true);
  _getTransactionsItem->setEnabled(true);
  _showTransactionsItem->setEnabled(true);
  _printTransactionsItem->setEnabled(true);
  _exportTransactionsItem->setEnabled(true);
  _singleTransferItem->setEnabled(true);
  _datedTransferItem->setEnabled(true);
  _transferTemplateItem->setEnabled(true);
  _standingOrderItem->setEnabled(true);
  _sendNowItem->setEnabled(true);
  _hbciItem->setEnabled(true);
  _ChangeLoginPasswordItem->setEnabled(true);
  _ChangeHbciPasswordItem->setEnabled(true);
  _CreateBackupMediumItem->setEnabled(true);
  _ToolboxItem->setEnabled(true);
  _ShowDataItem->setEnabled(true);
  if(_hardwareItem) _hardwareItem->setEnabled(true);
  _helpItem->setEnabled(true);
  _updateItem->setEnabled(true);
  _exitItem->setEnabled(true);
  _knownBanksItem->setEnabled(true);
  _jobQueueItem->setEnabled(true);
  
 slotRefresh();
}

/** Read property of HoBIT_Connector m_Connector. 
*/
HoBIT_Connector * GuiMain::getConnector()const{
	return _connector;
}

/**
*shows welcome widget 
*/
void GuiMain::slotShowWelcome(const QString & smallHelpFile){
 textLabelStatus->setText("Welcome"); 
 slotRaiseWidget(WELCOME_WIDGET);
 slotShowSmallHelp(smallHelpFile); //override the settings made by slotRaiseWidget
}

/** Loads the configuration files. 
*/
bool GuiMain::loadEnvironment(){
   setCursor(Qt::WaitCursor);
   bool result = true;
   if(decryptData())
     result = _hobit->loadEnvironment(); 
   else result = false;  
  if(result && USE_LIVE_CD_FEATURES) 
    slotRestoreOsConfig(); //this may fail, however it is not critical
 
  if(result)
   slotRefresh();   
  unsetCursor();
  // added as there's a permanent wait cursor on OSX  (AG)
  setCursor(QCursor(Qt::ArrowCursor));  
  return result;
}
/** Adds a log text to the log text widget
*/
void GuiMain::slotAddLogText(QString logText){
 _guilog->addLogText(logText);
}

/** Opens a dialog which asks the user wether the stores jobs should be
* send or should not. If the user answers "Yes" the Job Queue will be
*executed. Otherwise The jobs will be stored for later use.
*/
int GuiMain::askForSend(){
 QString caption = tr("Send jobs now?");
 QString question = tr("What would you like MoneyPenny to do?");
 QString later = tr("&Add to job queue \nfor sending later");
 QString now = tr("&Send immediately");
// QString nowAndGetTransactions = tr("Send immediately and \nget &transactions and balance");
 int result = 
  QMessageBox::question
  (  this, caption, question, later, now);//, nowAndGetTransactions);
     //this is considered to be too much asking
/* if (!result) QMessageBox::information
   ( this,
     tr("Job Queue"),
     tr("New Jobs have been queued. \nPlease do not forget to send them later! ") );
*/     
 return result;
}

/** Read property of QString _ConfigDir. 
*/
const QString& GuiMain::getConfigDataDir()const{
  return _ConfigDataDir;
}

/** Write property of QString _ConfigDataDir. 
*/
void GuiMain::setConfigDataDir( const QString& _newVal){
  _ConfigDataDir=_newVal;
}  

/** Write property of QString _CryptDir. 
*/
void GuiMain::setCryptDir( const QString& _newVal){
  _CryptDir = _newVal;
  if(_Crypt)
    _Crypt->setCryptDir(_newVal);
  if(_hobit)  
    _hobit->setCryptDir(_newVal);
  if(_guihbciwizard)
    _guihbciwizard->setMediumDir(_newVal); //initialize wizard with the same dir. User may modify this later
}  


/** Read property of QString _CryptDir. 
*/
const QString& GuiMain::getCryptDir(){
  return _Crypt->getCryptDir();
}  

/**
 *Raises a transfer template
 */
 void GuiMain::slotRaiseTemplate( int id, 
                                  QString name, 
                                  QString accountId, 
                                  QString bankId, 
                                  QString value, 
                                  QStringList purpose){
 
 if (id != SINGLE_TRANSFER_WIDGET && id != DATED_SINGLE_TRANSFER_WIDGET && id != START_STANDING_ORDER_WIDGET){
   QMessageBox::critical(widgetStack, "problem:", "could not create transfer from template");
 }
 else{
  
    switch (id){
    
        case SINGLE_TRANSFER_WIDGET:  
        
               _guisingletransfer->lineEditOtherName->setText(name);
               _guisingletransfer->lineEditOtherAccountId->setText(accountId);
               _guisingletransfer->lineEditOtherBankCode->setText(bankId);
               _guisingletransfer->lineEditValue->setText(value);
               _guisingletransfer->setPurpose(purpose);
               slotRaiseWidget(SINGLE_TRANSFER_WIDGET);
               break;
                       
        
        case DATED_SINGLE_TRANSFER_WIDGET:
        
               _guidatedsingletransfer->lineEditOtherName->setText(name);
               _guidatedsingletransfer->lineEditOtherAccountId->setText(accountId);
               _guidatedsingletransfer->lineEditOtherBankCode->setText(bankId);
               _guidatedsingletransfer->lineEditValue->setText(value);
               _guidatedsingletransfer->setPurpose(purpose);
               slotRaiseWidget(DATED_SINGLE_TRANSFER_WIDGET);
               break;
        
        
               break;
        
        case START_STANDING_ORDER_WIDGET:
        
               _guistartstandingorder->lineEditOtherName->setText(name);
               _guistartstandingorder->lineEditOtherAccountId->setText(accountId);
               _guistartstandingorder->lineEditOtherBankCode->setText(bankId);
               _guistartstandingorder->lineEditValue->setText(value);
               _guistartstandingorder->setPurpose(purpose);
               slotRaiseWidget(START_STANDING_ORDER_WIDGET);
               break;
        
       default:  break;
    }//end switch
 }//end else
}
 

/** Write property of Storage _Storage. 
*/
void GuiMain::setStorage( const StorageType& _newVal){
   _Storage = _newVal;
   if(_guihbciwizard != NULL)
     _guihbciwizard->setStorageType(_Storage);
}

/** Presently not needed
*/
void GuiMain::showEvent(QShowEvent * se){
}
 
/** Run a test connection to www.google.de or another url
*/
void GuiMain::slotTestInternet(const QString & url, int port, ConnectionState * state){
 
 QString oldUrl = _connector->getUrl();
 int oldPort = _connector ->getPort();
 
 ConnectionState oldState = _connector->connectionState();
 if(oldState == ConnectionConnected) 
   _connector->closeConnection();  
 
 _connector->setUrl(url);
 _connector->setPort(port);
 _connector->connectTCP();
 * state = _connector->connectionState();
 if( * state == ConnectionConnected){
   _connector->closeConnection();
   * state = ConnectionWasOk;  
   QMessageBox::information(this, tr("Internet Connection Test"), 
     tr("Your internet setup seems to be Ok. Could connect to: ") + url + tr(" : Port: " + QString::number(port)));
 }
 else QMessageBox::warning(this, tr("Internet Connection Test"), 
       tr("Your internet setup does not work properly. Connection test failed. <br>") +
       tr("Tried to connect to: ") + url + tr(" : Port: " + QString::number(port) ));
 slotTimerDone(); //update Status Bar
 _connector->setUrl(oldUrl);
 _connector->setPort(oldPort);      
}


/** Read property of QString _MountPoint. 
*/
const QString& GuiMain::getMountPoint()const{
	return _MountPoint;
}

/** Write property of QString _MountPoint. 
*/
void GuiMain::setMountPoint( const QString& _newVal){
	_MountPoint = _newVal;
}

/** restart the application
*/
void GuiMain::slotRestart(){
   _exitCode = EXITCODE_RESTART;	
   slotClose();
}


/** shows information about a single transaction
*/

void GuiMain::slotShowTransactionInfo(Pointer <Transaction> trc){
   _guitransactioninfo->showTransaction(trc);
   slotRaiseWidget(TRANSACTION_INFORMATION_WIDGET);
}

/** shows information about a single transaction
*/
void GuiMain::slotShowStandingOrderInfo(Pointer<StandingOrder> sto){
   _guitransactioninfo->showStandingOrder(sto);
   slotRaiseWidget(TRANSACTION_INFORMATION_WIDGET);
}
/** shows information about a single dated transaction
*/
void GuiMain::slotShowDatedTransferInfo(Pointer<DatedTransfer> dt){
   _guitransactioninfo->showDatedTransfer(dt);
   slotRaiseWidget(TRANSACTION_INFORMATION_WIDGET);
}
/** Import an RDH2 formatted key. 
*/
void GuiMain::slotImportRDH2Key(const QString & fileName){
  QMessageBox::information(this, QString("MoneyPenny"), // ask user for password
                                    QString(tr("This function is not yet implemented completely!")));
/*   bool ok;
 
   QString password=QInputDialog::getText(QString("MoneyPenny"), // ask user for password
                                    QString(tr("Please input the password for the keyfile you want to import!")),
                                    QLineEdit::Password,
                                    0,
                                    &ok);
  if(ok)
   _hobit->importRDH2Key(fileName, password);
  _hobit->importRDH2Key(fileName, "Password");*/ // To be removed!!!!
}


/** Performed after HBCI Wizard has finished
*/
void GuiMain::slotSetupSuccessful(HBCI_MODE mode){
slotActivateMenuFunctions();
if (mode == HBCI_MODE_RDH)
 slotShowIniLetter();
else slotAccountLogin();
}


/** shows the splash screen */
  void GuiMain::showSplashScreen(){
    if(!_guiSplash->isVisible())
      _guiSplash->showFullScreen();
    hide(); //hide guimain
}

/** hides the splash screen */
  void GuiMain::hideSplashScreen(){
  _guiSplash->hide();
  show(); //show main
}

/** returns the splash screen */
  GuiSplash * GuiMain::getSplashScreen(){
  return _guiSplash;
}

/**Search for the removable medium*/
 bool GuiMain::findMedium(){
  bool mediumOk = _guiFindMedium->exec() == QDialog::Accepted;
  if(mediumOk){
     createConfigDir();
     renewHoBIT();
     setMountPoint(_guiFindMedium->getMountPoint());
     setCryptDir(_guiFindMedium->getCryptDirName()); // sets directory in _Crypt
     setStorage(_guiFindMedium->getStorage());
     return true;
  }
  else {
     QMessageBox::warning(_guiSplash, QObject::tr("Error"),QObject::tr("Can not use removable medium, aborting."));
     return false;
  }
 }

 /**ask the user for the local data password.*/
void GuiMain::login() {
  _guiLoginPassword->checkForPasswordFile();
  bool passwordOk = _guiLoginPassword->exec() == QDialog::Accepted;
   if(!passwordOk)
  exit (2);
}

/** deletes the old hobit object and creates a new one */
void GuiMain::renewHoBIT(){
   if(_hobit)
     delete _hobit;
  _hobit = new HoBIT( false,
                      this,
                      MONEYPENNY_OPENHBCI_FILE,
                      MONEYPENNY_AQMONEY_FILE,
		      MONEYPENNY_TEMPLATE_FILE,
                      MONEYPENNY_HBCIDATA_DIR,
                      _ConfigDataDir,
                      APPLICATION_NAME,
                      _connector);

  _guitools->setHoBIT(_hobit);
  _GuiShowData->setHoBIT(_hobit);
  _guiaccountinformation->setHoBIT(_hobit);
  _guitransactions->setHoBIT(_hobit);
  _guigettransactions->setHoBIT(_hobit);
  _guiiniletter->setHoBIT(_hobit);
  _guisingletransfer->setHoBIT(_hobit);
  _guidatedsingletransfer->setHoBIT(_hobit);
  _guidatedtransfers->setHoBIT(_hobit);
  _guistandingorders->setHoBIT(_hobit);
  _guistartstandingorder->setHoBIT(_hobit);
  _guiknownbanks->setHoBIT(_hobit);
  _guilogin->setHoBIT(_hobit);
  _guihbciwizard->setHoBIT(_hobit);
  _guichangeaccountname->setHoBIT(_hobit);
  _guijobs->setHoBIT(_hobit);
  _guiaddaccount->setHoBIT(_hobit);
  _guitransfertemplates->setHoBIT(_hobit);
  _guitransactioninfo->setHoBIT(_hobit);

  connect(_hobit, SIGNAL(logTextToShow(QString)), this, SLOT(slotAddLogText(QString)));
  connect(_hobit->getAccountJobFactory(), SIGNAL(logTextToShow(QString)), this, SLOT(slotAddLogText(QString)));
  connect(_hobit->getAdminJobFactory(), SIGNAL(logTextToShow(QString)), this, SLOT(slotAddLogText(QString)));

  _hobit->loadEnvironment();
  _guihbciwizard->updateAccountList();
 slotRefresh();

}

/** shows data export dialog */
  void GuiMain::slotExport(QTable* table){
   _guidataexport->setTableToExport(table);
   _guidataexport->setMountPoint(_MountPoint.latin1());

    slotRaiseWidget(EXPORT_WIDGET);
 }
 
/** Shows a dialog to change the login password.
*/
void GuiMain::slotChangeLoginPassword(){
   if(_guiLoginPassword->checkForPasswordFile()){
     QMessageBox::warning(this, tr("Warning"), tr("Could not read your password verification file."));
   }
   bool passwordOk = _guiLoginPassword->exec() == QDialog::Accepted;
   if(!passwordOk){
     QMessageBox::critical(this, tr("Error"), tr("Wrong Password!"));
   }
   else{
     _guiLoginPassword->switchToEditPasswordMode();
     passwordOk = _guiLoginPassword->exec() == QDialog::Accepted; //changes password in _Crypt
     if(passwordOk){
       if(encryptData()){
         QMessageBox::information(this, tr("Information"), tr("Your login password has been changed."));
       } 
       else{                 
         QMessageBox::critical(this, tr("Error"), tr("Could not save your data. Please make shure, that your medium is writeable and try again."));
      }
     }  
   }
}

/** Shows a dialog to change the HBCI password.
*/
void GuiMain::slotChangeHbciPassword(){  
   slotRaiseWidget(HBCI_WIZARD);
   _hobit->saveEnvironment(); //save all data first
   encryptData();
   _guihbciwizard->changePassword();
}

/** Shows a dialog to change the HBCI password.
*/
void GuiMain::slotCreateBackupMedium(){ 
    bool ok = true;
    KeyFileOpenHbci * keyfile = NULL;
    QString mediumName = "No Medium";
    slotRaiseWidget(WELCOME_WIDGET);
    
    if(_hobit->getHbciMode() == (HBCI_MODE)HBCI_SECURITY_RDH){
     QString password = QInputDialog::getText(tr("Reading Keyfile for Backup"), tr("Please enter the password for your keyfile."),
      QLineEdit::Password , "", &ok, this, "PasswordDlg"); 
      Pointer<Medium> medium = _hobit->getMedium();
      if(medium.isValid())
       mediumName = medium.ref().mediumName().c_str();      
      keyfile = new KeyFileOpenHbci(mediumName, password);
     if(!ok || !keyfile->readFile()){
       QMessageBox::critical(this, tr("Error"), tr("Could not read keyfile: ") + mediumName + tr("\n Wrong password or wrong file format?"));
       delete keyfile;
       keyfile = NULL;
       return;
     }
    }
    
    ok = QMessageBox::information(this, tr("Creating Backup Medium"), tr("Please remove your working medium now."),
                                  QMessageBox::Ok, QMessageBox::Cancel)== QMessageBox::Ok; 
    if(ok && findMedium()){  
     ok = _hobit->saveEnvironment() && encryptData(); // this will also create the directory 
     if(ok && keyfile)
       ok = keyfile->writeFile(mediumName);
   }
   if (ok){
       QMessageBox::information(this, tr("Creating Backup Medium"), 
                                      tr("Your backup files have been written.") +"<br>" +
                                      tr("Please wait until the indicator lamp of the drive (or USB device) is off.")+"<br>" +
                                      tr("Then remove the backup medium and reinsert your working medium when asked for it."),
                                  QMessageBox::Ok); 
    findMedium();
  }
  if(keyfile)
    delete keyfile;
}


/** Shows a dialog to send an SQL-Query
*/
void GuiMain::slotSqlQuery(){ 
  slotRaiseWidget(SQL_QUERY_WIDGET);  
}

/** Executes the sql query
*/
void GuiMain::slotExecuteSqlQuery(QString query){  
   
  
  // get Home directory path
  QString path = getenv("HOME");
  path += "/";
  path += MONEYPENNY_MAIN_DIR;

  QDir directory(path);
  if (directory.exists()){
	   path += "/";
	   path += MONEYPENNY_SQL_FILE_NAME;       
          // build the database
           slotMakeSQL(path);
 
    }
    else qDebug ("MoneyPenny configdatadir not found. Can not create an SQL file.");
	   	 
 
    slotRaiseWidget(SQL_RESULT_WIDGET);
    _guisqlqueryresult->executeQuery(query);
}

/** Get a new account list from bank server.
*/
void GuiMain::slotUpdateAccountList(){
    _hobit->createJobGetAccounts();
    slotSend();
}
/** Sets _AutoShutdown Mode. 
*/
void GuiMain::slotSetShutdownMode(bool autoShutdown){
 _AutoShutdown = autoShutdown;
}

/** Calls the printer setup program
*/
void GuiMain::slotAddPrinter(){
/*  if(_AddPrinterProcess)
    delete _AddPrinterProcess;
  _AddPrinterProcess = new QProcess(this);
  _AddPrinterProcess->addArgument("sudo");
  _AddPrinterProcess->addArgument("gnome-cups-manager");
  _AddPrinterProcess->start();
  */
  slotRaiseWidget(HARDWARE_WIDGET);
}

/** Stores the knoppix configuration.
*/
bool GuiMain::slotStoreOsConfig(){
  qDebug("GuiMain::slotStoreOsConfig");
  if (USE_LIVE_CD_FEATURES)
  { //save OS config as knoppix.sh to ConfigDataDir
   
     //No automatic saving! Ask the user, if he wants the config to be saved!!!
     if(QMessageBox::question(this, tr("Save Hardware Configuration Security Request"), 
        tr("You called the hardware configuration page during this session. <br> Do you want to save the printer and internet configuration you setup?") +
        tr("MoneyPenny will restore a saved configuration automatically on next startup."), 
        tr("Yes"), tr("No")) == 0) //First button "Yes" was clicked
       {
           QString dirName = _ConfigDataDir + "/" + MONEYPENNY_OS_CONFIG_DIR;
           QDir dir(dirName);
           if(!dir.exists())
             dir.mkdir(dirName);
           QString command = "sudo /usr/sbin/saveconfig-moneypenny " + dirName;  
           qDebug("Saving OS configuration with command: %s", command.latin1());                  
           system(command.latin1()); 
           int cryptError = _Crypt->encryptArchive(MONEYPENNY_OS_CONFIG_DIR, 
                                                   _ConfigDataDir,            
                                                   getCryptDir() + "/" + MONEYPENNY_ENCRYPTED_KNOPPIX_FILE,
                                                   getCryptDir() + "/" + MONEYPENNY_KNOPPIX_INIVECTOR_FILE); 
           if(cryptError){
            QMessageBox::information(this, tr("Error"), tr("Failed to encrypt and save hardware configuration."));
            qDebug("GuiMain::closeEvent: Failed to encrypt and save knoppix files. Error code: %i", cryptError);
           }                                                                                     
       }
       else QMessageBox::information(this, tr("Skipping Save Configuration"), tr("Setup changes will be lost."));  
       return true;
  }//end if LIVE_CD
  else return false;
}

/** Decrypts the crypted knoppix configuartion directory and then calls the knoppix restore script knoppix.sh.
*/
int GuiMain::slotRestoreOsConfig(){
  if(! USE_LIVE_CD_FEATURES) // just for safety
    return 0;

  int result = 0;  
  QString cryptedConfigFileName = getCryptDir() + "/" + MONEYPENNY_ENCRYPTED_KNOPPIX_FILE;
  QString ivFileName = getCryptDir() + "/" + MONEYPENNY_KNOPPIX_INIVECTOR_FILE;
  QString configFileDir = _ConfigDataDir + "/" + MONEYPENNY_OS_CONFIG_DIR;
  QFile knoppixFile(cryptedConfigFileName);
  if(!knoppixFile.exists()){
     result = -1;
     qDebug("No Knoppix configuration available. Could not find: %s ", cryptedConfigFileName.latin1());
     startCupsd();    //Start cups daemon, because there is no script for starting it
     return result;
   } 
  result = _Crypt->decryptArchive(cryptedConfigFileName, _ConfigDataDir, ivFileName);  
  if(result != 0){
    result = -2;
    QMessageBox::critical(this, tr("Error"), 
       tr("Decrypting Knoppix configuration failed. Most probably the file is damaged. \n Could not decrypt: " + cryptedConfigFileName));
       qDebug(QString("Decrypting Knoppix configuration failed. Could not decrypt: " + cryptedConfigFileName).latin1());
    return result;
   } 
  QFile script( configFileDir + "/knoppix.sh");
  if(!script.exists()){
    result = -3;
    qDebug("Error: Can not restore OS-config. Could not find script: %s", script.name().latin1());
    startCupsd();    //Start cups daemon, because the script can't do it
    return result;
  } 
  else{
    QString chmodCommand = "sudo chmod u+x " + script.name();  
    QString restoreCommand = "sudo " + script.name() + " "  + configFileDir;// use configFileDir as $1
    system (chmodCommand.latin1());
    qDebug("Restoring OS-config using command: %s", restoreCommand.latin1());
    if (system(restoreCommand) != 0) //this will also start cupsd
     result = -4;
    startCupsd();    //Start cups daemon, if the script did not do it
  }
  return result;
}

/** generate sql database */
void GuiMain::slotMakeSQL(QString filename){

 list<Pointer<Account> > accounts = _hobit->listAccounts();
// get a list of all accounts managed by moneypenny
 list<Pointer<Account> >::iterator account;


// create list of all transactions
list<Pointer<Transaction> > transactions;

// create list of all transactions
list<Pointer<DatedTransfer> > datedtransfers;

// create list of all transactions
list<Pointer<StandingOrder> > standingorders;

   // now get transaction lists for all accounts
   for (account = accounts.begin(); account != accounts.end(); account++) {

	// get Account ID and SubID
    	QString accountId = account->ref().accountId();


	// create list of all transactions
	list<Pointer<Transaction> > currenttransactions = _hobit->listTransactions(  0,
                            			       				    "*",
                                        					    accountId.ascii(),
                                        					    "*",
                                        					    QDate(1980, 1,1),
                                        					    QDate::currentDate().addYears(1));

	// create list of all transactions
	list<Pointer<DatedTransfer> > currentdatedtransfers = _hobit->listDatedTransfers(0, 
											"*",
											 accountId.ascii(),
											 "*");

	// create list of all transactions
	list<Pointer<StandingOrder> > currentstandingorders =  _hobit->listStandingOrders(  0, 
											   "*",
											   accountId.ascii(),
											    "*"); 


	// merge transactions of all accounts to one large list
	list<Pointer<Transaction> >::iterator transit;
        for (transit = currenttransactions.begin(); transit != currenttransactions.end(); transit++){
	transactions.push_back(*transit);
	}
	
	
	// merge dated transfers of all accounts to one large list
	list<Pointer<DatedTransfer> >::iterator datit;
	for (datit = currentdatedtransfers.begin(); datit != currentdatedtransfers.end(); datit++){
	datedtransfers.push_back(*datit);
	}

	
	// merge standing orders of all accounts to one large list
	list<Pointer<StandingOrder> >::iterator standit;
	for (standit = currentstandingorders.begin(); standit != currentstandingorders.end(); standit++){
	standingorders.push_back(*standit);
	}	
        
	_sql->setAll(transactions, datedtransfers, standingorders);
	_sql->writeSqlFile(filename); 
	 }
  } 
	
/** Starts the cups printer daemon.
*/
bool GuiMain::startCupsd(){
    if(system ("ps -A | grep cupsd") == 0) //cupsd is already running
      return true;
    QString cupsName = "/etc/init.d/cupsys"; //this is the startup skript in Knoppix
    QString startCommand = "sudo " + cupsName + " start";
    QFileInfo cupsFileInfo (cupsName);
    if (cupsFileInfo.isExecutable()){
      return system (startCommand.latin1())== 0;
    }  
    else {
      qDebug ("Can not start cups daemon using command: %s", startCommand.latin1());  
      return false;
    }  
}
/** Output of window size
*/
void GuiMain::resizeEvent (QResizeEvent * re){
  QString s = QString::number(re->size().width()) +" : " + QString::number(re->size().height());
  qDebug("Window resized to: %s", s.latin1());
}

/** Decrypts the data on the removeable medium and copies them to /home/moneypenny_data
*/
bool GuiMain::decryptData(){
    QString bankName =  _guiFindMedium->getBankName();
    int result =  _Crypt->decryptArchive(_CryptDir + "/" + bankName + MONEYPENNY_ENCRYPTED_HBCI_CONF_FILE, 
                                         _ConfigDataDir, 
                                         _CryptDir + "/" + bankName + MONEYPENNY_HBCI_INIVECTOR_FILE);
    
    if(result < -1){
      QMessageBox::critical(this, QObject::tr("Fatal Error"), QObject::tr("Failed to decrypt data. Aborting") );
      return false;
    }
    else return true;
}

/** Crypts the data in /home/moneypenny_data and copies them to the removeable medium.
*/
bool GuiMain::encryptData(){
     QDir testDir(_CryptDir);
     if(!testDir.exists())
      if(!testDir.mkdir(_CryptDir)){ //try to make the directory
        qDebug("GuiMain::encryptData: Failed to create crypt directory: %s", _CryptDir.latin1());
   	QMessageBox::critical(this, tr("Error"),tr("Could not create directory:") + " " + _CryptDir);
        return false;
      }  
    QString bankName = _hobit->getBankNameForId(_hobit->getBankId());
    if(bankName.isEmpty())
     bankName = "MyBank";
    int val = _Crypt->encryptArchive(QString( MONEYPENNY_HBCIDATA_DIR) + " "
                                  + MONEYPENNY_AQMONEY_FILE + " "
                                  + MONEYPENNY_OPENHBCI_FILE + " "
                                  + MONEYPENNY_TEMPLATE_FILE,
                                  _ConfigDataDir,
                                  _CryptDir + "/" + bankName + "_" + MONEYPENNY_ENCRYPTED_HBCI_CONF_FILE,
                                  _CryptDir + "/" + bankName + "_" + MONEYPENNY_HBCI_INIVECTOR_FILE); 
    
    if (val != 0) { 
   	QMessageBox::critical(this, tr("Error"),tr("Error when encrypting data."));
	return false;
    }
    return true;
}

/** Sets the timeOut value in _Connector
*/
void GuiMain::slotSetTimeOut(int newTimeOut){
 if(newTimeOut != _connector->getTimeOut())
   _connector->setTimeOut(newTimeOut);
}

/** Asks the user, whether he wants to close withouts saving.
*/
void GuiMain::reject(){
 if(QMessageBox::question(this,
    tr("Confirmation"),
    tr("Do you want to close MoneyPenny without saving your data?"),
    tr("Yes"), //button 0 text
    tr("No"),  //button 1 text
    "",  //button 2 text
    1)         //default button
    == 0) // button 0 selected
    {
       _hobit->removeUncryptedEnvironment();  
       GuiMainBase::reject();
    }
}
/**
*Raises the toolbox widget 
*/
void GuiMain::slotShowData()
{
  slotRaiseWidget(SHOWDATA_WIDGET);
  textLabelStatus->setText(tr("Show User Data"));
  //slotShowSmallHelp("smallShowData.html");
  slotRefresh();
}

/**
*Displays update Information.
*/
void GuiMain::slotUpdateProgram()
{
  QString msg = tr("This is MoneyPenny Version: ") + VERSION + "<br>";
  msg += tr("It was released on: ") + MONEYPENNY_RELEASE_DATE + "<br>";
  if(USE_LIVE_CD_FEATURES)
   msg += tr("The operating system on the CD is: ") + MONEYPENNY_CD_OS_VERSION + "<br>"; 
  msg += tr("Please have a look at http://www-in.fh-swf.de/moneypenny for availability of updates.") +"<br>";
 _guiwelcome->displayMessage(msg);
 slotRaiseWidget(WELCOME_WIDGET);
}

/** Raises the printer driver setup widget.
*/
void GuiMain::slotBuildPrinterDriverDatabase(){
 slotRaiseWidget(PRINTER_DRIVER_TO_XML_WIDGET);
}

/**Deletes OpenHbci-LogFiles
*/
void GuiMain::slotDeleteLogFiles(){
 QString logDir = _ConfigDataDir + "/" + MONEYPENNY_HBCIDATA_DIR + "/" 
                  + QString::number(MONEYPENNY_DEFAULT_COUNTRY) + "/" + _hobit->getBankId() + "/logs";
 if( (_Storage == STORAGE_FLOPPY) || (QMessageBox::question(this, tr("Confirmation"), tr("Really delete all *.log files in:") +"\n" + logDir,
  QMessageBox::Yes, QMessageBox::No) == QMessageBox::Yes)){
  QString rmCommand = "rm " + logDir + "/*.log";
  qDebug ("Deleting all log files. Command: %s", rmCommand.latin1());
  system (rmCommand);
 } 
}

/**Shuts down current session and calls HBCI wizard to setup a new bank.
*/
void GuiMain::slotAddBank(){
 if(QMessageBox::question(this, tr("Add New Bank"), 
   tr("MoneyPenny will now save your data and create an empty dataset for a new bank.") + "\n"
   +tr("Do you want to continue?"), 
   QMessageBox::Cancel, QMessageBox::Ok | QMessageBox::Default
 )!= QMessageBox::Ok)
  return;
 
 slotRaiseWidget(WELCOME_WIDGET);  
 textLabelStatus->setText(tr("Saving Data"));
 qApp->processEvents();
 
 if(_hobit->saveEnvironment()){
   if(encryptData()){
    _hobit->removeUncryptedEnvironment();
    createConfigDir();
    renewHoBIT();
    textLabelStatus->setText(tr("Empty dataset created"));
    slotAccountLogin();
   } 
 }
}

/**Shuts down current session and runs the select-bank loop.
*/
void GuiMain::slotHbciSetupAborted(){
    _exitCode = EXITCODE_RESTART_SESSION;
    close();
}

