/***************************************************************************
                          guistartstandingorder.cpp  -  description
                             -------------------
    begin                : Sa Nov 13 2004
    copyright            : (C) 2004 by Marcel Mulch
    email                : marcel@mulch-online.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "guistartstandingorder.h"
#include <qcombobox.h>
#include <qdatetime.h>
#include <qlineedit.h>
#include <qstring.h>
#include <qmessagebox.h>
#include <qdatetimeedit.h>
#include <qspinbox.h>
#include <qlayout.h>
#include <qpushbutton.h>
#include <qframe.h>
#include <qlabel.h>
#include <qvalidator.h>


/**
*constructor of GuiStartStandingOrder
*
*@param main a pointer on moneypenny´s main class
*@param parent a pointer on the main widgets widget stack
*@param name widgets title
*@param hobit the hobit object, needed for hbci functions
*/
GuiStartStandingOrder::GuiStartStandingOrder(QWidget *parent, const char *name, HoBIT* hobit )
:GuiTransferForm(parent, name, hobit) {
  formNameLabel->setText(tr("Standing Order"));
  _ReconfirmationHeader = tr("You are about to create the following standing order:");
  _FormType =   START_STANDING_ORDER_WIDGET; 
  refresh();
}


/**
*destructor of GuiStartStandingOrder
*/
GuiStartStandingOrder::~GuiStartStandingOrder(){
qDebug("Destructor GuiStartStandingOrder finished");
}




/** 
*called function after pressing button of formular,
*gets current inputs of formular for transferjob  
*/
void GuiStartStandingOrder::accept(){
  _FirstDate = dateEditFirst->date();
  _LastDate = dateEditLast->date();
  
  _Day = spinBoxDay->value();
  
  _Weekly = false;
  _Turnus = 99;
  
  switch(comboBoxTurnus->currentItem()){
  
    case 0:           _Weekly = true;
                      _Turnus = 1;
                      break;
    case 1:           _Weekly = false;
                      _Turnus = 1;
                      break;
    case 2:           _Weekly = false;
                      _Turnus = 3;
                      break;
    case 3:           _Weekly = false;
                      _Turnus = 6;
                      break;
    case 4:           _Weekly = false;
                      _Turnus = 12;
                      break;
    default:          QMessageBox::critical(this, tr("Error"), tr("Error in execution interval selection!")); 
                      break;
  }
  GuiTransferForm::accept();

/*  if(! isOk())
   return;

  if (! userReconfirmation()) 
    return;   

  bool jobSupported =  HoBIT_Bank::supportsJob(_hobit->getConfigDataDir() + "/" + MONEYPENNY_OPENHBCI_FILE, "HIDAES")
                     && HoBIT_Bank::supportsJob(_hobit->getConfigDataDir() + "/" + MONEYPENNY_OPENHBCI_FILE, "HIDABS");      
  if(! jobSupported){
     QMessageBox::critical(this, tr("Error"), tr("Sorry, your bank does not support standing orders."));
     return;
  } 
  
  QString ourAccount = _hobit->getCurrentAccount();
  QString name = lineEditOtherName->text();
  QString bankCode = lineEditOtherBankCode->text();
  QString accountId = lineEditOtherAccountId->text();
  QString value = lineEditValue->text();
  QDate firstD = dateEditFirst->date();
  QDate lastD = dateEditLast->date();
  Date firstdate(firstD.day(), firstD.month(), firstD.year());
  Date lastdate(lastD.day(), lastD.month(), lastD.year());
  
  int day = spinBoxDay->value();
  
  bool weekly = false;
  int turnus = 99;
  
  switch(comboBoxTurnus->currentItem()){
  
    case 0:           weekly = true;
                          turnus = 1;
                          break;
    case 1:           weekly = false;
                          turnus = 1;
                          break;
    case 2:           weekly = false;
                          turnus = 3;
                          break;
    case 3:           weekly = false;
                          turnus = 6;
                          break;
    case 4:           weekly = false;
                          turnus = 12;
                          break;
    default:          QMessageBox::critical(this, tr("Error"), tr("Error in execution interval selection!")); 
                      break;
  }
  
  
  Value val(value.ascii(), "EUR");
  
  list<string> oname;
  oname.push_back(name.ascii());

  list<string> purpose;
  HoBIT_String::copyQStringList(&_PurposeList, & purpose);
  
  int err = _hobit->createJobStartStandingOrder( SKIP_FALSE_TRANSACTIONS,  // skipbad
                                                 MONEYPENNY_DEFAULT_COUNTRY,                 // country
                                                 "*",                 // bankcode
                                                 ourAccount.ascii(),  // accountid
                                                 "", 		   // accountSubId
                                                 MONEYPENNY_DEFAULT_COUNTRY,                 // other country
                                                 bankCode.ascii(),    // other bankcode
                                                 accountId.ascii(),   // other account
                                                 oname,               // other name
                                                 purpose,             // purpose
                                                 val,                 // value
                                                 53, 	           // textkey
                                                 firstdate,           // normal date
                                                 firstdate,           // first date
                                                 lastdate,            // last date
                                                 turnus,              // turnus
                                                 weekly,              // weekly
                                                 day);                // day
                  
  if(err != HobitErrorSuccess)
    qDebug("GuiStartStandingOrder::accept: Error creating JobStartStandingOrder. Error Code: %i", err);
  _hobit->createJobGetStandingOrders();     
  emit jobCreated();
  emit raiseWidget(STANDING_ORDERS_WIDGET);
 */ 
}

/** Builds the html table rows for the reconfirmation dialog. Returns a string that has to be included into <table> </table> tags. 
*/
 QString GuiStartStandingOrder::reconfirmationTableRows(){
   QString result = GuiTransferForm::reconfirmationTableRows();
   result += "</tr> <tr>\n";
   
   result += "<tr>";
   result +=  "<td> " + tr("First Execution Date:") + "</td>  ";
   result +=  "<td> " + dateEditFirst->date().toString(Qt::LocalDate) + "</td>  ";
   result += "</tr>\n";
   
   result += "<tr>";
   result +=  "<td> " + tr("Last Execution Date:") + "</td>  ";
   result +=  "<td> " + dateEditLast->date().toString(Qt::LocalDate) + "</td>  ";
   result += "</tr>\n";
   
   result += "<tr>";
   result +=  "<td> " + tr("Interval:") + "</td>  ";
   result +=  "<td> " + comboBoxTurnus->currentText() + "</td>  ";
   result += "</tr>\n";
   return result;
 }

/** Creates the standing order
 */
bool GuiStartStandingOrder::createJob(){
  bool jobSupported =  HoBIT_Bank::supportsJob(_hobit->getConfigDataDir() + "/" + MONEYPENNY_OPENHBCI_FILE, "HIDAES")
                     && HoBIT_Bank::supportsJob(_hobit->getConfigDataDir() + "/" + MONEYPENNY_OPENHBCI_FILE, "HIDABS");      
  if(! jobSupported){
     QMessageBox::critical(this, tr("Error"), tr("Sorry, your bank does not support standing orders."));
     return false;
  } 
  
  list <string> otherName;
  otherName.push_back(_OtherName);
  list<string> purpose;
  Date firstDate(_FirstDate.day(), _FirstDate.month(), _FirstDate.year());
  Date lastDate(_LastDate.day(), _LastDate.month(), _LastDate.year());
  HoBIT_String::copyQStringList(&_PurposeList, & purpose);
  
  int err = _hobit->createJobStartStandingOrder( SKIP_FALSE_TRANSACTIONS,  // skipbad
                                                 MONEYPENNY_DEFAULT_COUNTRY,                 // country
                                                 "*",                 // bankcode
                                                 _OurAccount.ascii(),  // accountid
                                                 "",           // accountSubId
                                                 MONEYPENNY_DEFAULT_COUNTRY,                 // other country
                                                 _OtherBankCode.ascii(),    // other bankcode
                                                 _OtherAccountId.ascii(),   // other account
                                                 otherName,               // other name
                                                 purpose,             // purpose
                                                 _Value,                 // value
                                                 53,               // textkey
                                                 firstDate,           // normal date
                                                 firstDate,           // first date
                                                 lastDate,            // last date
                                                 _Turnus,              // turnus
                                                 _Weekly,              // weekly
                                                 _Day);                // day
   if(err != HobitErrorSuccess){
     QMessageBox::critical(this, tr("Error"), tr("Creating the job failed."));
     qDebug("GuiStartStandingOrder::accept: Error creating JobStandingOrder. Error Code: %i", err);
     return false;  
   }
  
   _hobit->createJobGetStandingOrders(); 
   return true; 
                                                   
}
