/***************************************************************************
                          guiconnections.cpp  -  description
                             -------------------
    begin                : Do Nov 4 2004
    copyright            : (C) 2004 by Marcel Mulch, Walter Roth
    email                : marcel@mulch-online.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "guitools.h"
#include "globals.h"

#include <qpushbutton.h>
#include <qheader.h>
#include <qvalidator.h>
#include <qfiledialog.h>
#include <qcheckbox.h> 

#include <qgroupbox.h>

/**
 *this is the constructor for create new user
 *
 *@param main
 *@param parent the parent dialog of this widget
 *@param name widgets title
 *@param hobit
 */
GuiTools::GuiTools(QWidget *parent, const char *name, HoBIT* hobit)
    :GuiToolsBase(parent, name)
{
  _hobit = hobit;
  _guicreateuser = new GuiCreateUser(this,  "Benutzer erstellen", _hobit);
   connect(_guicreateuser, SIGNAL(refreshRequest()), this, SIGNAL(refreshRequest()));
  _guichangeversion = new GuiHbciVersion(this,  "Hbci Version", _hobit);
   connect(_guichangeversion, SIGNAL(refreshRequest()), this, SIGNAL(refreshRequest()));

  //hide the table header
  //listViewConnections->header()->hide();
  QValidator* validator = new QIntValidator( 1, 65355, this );
  lineEditPort->setValidator(validator);

  // hide sql functions if not needed
  if (! USE_MYSQL_FEATURES) groupBoxDataAnalysis->hide();
  if(!USE_LIVE_CD_FEATURES){
    checkBoxShutdown->hide();
    buttonSaveKnoppix->hide();
  }
  if(USE_LIVE_CD_FEATURES || ! DEBUG_HARDWARE)
    buttonBuildPrinterDriverDatabase->hide(); //useless for a CD or a release on HDD
  qDebug("Constructor GuiTools");
}

GuiTools::~GuiTools()
{
  qDebug("Destructor GuiTools");
}

/**
* starts dialog for creating a new user
*/
void GuiTools::slotCreateUser()
{
    if(QMessageBox::question(this, tr("Confirmation"),
    tr("Delete present setup data and start a new setup?") + "\n"+
    tr("This will terminate your current session."),
    QMessageBox::Yes, QMessageBox::No) == QMessageBox::Yes)
      emit raiseWidget(HBCI_WIZARD);
}


/**
* Opens a Dialog to change the used HBCI version
*/
void GuiTools::slotChangeHbci()
{
  _guichangeversion->show();
  emit refreshRequest();
}

/**
* slot which retrieves the bank servers public keys 
*/
void GuiTools::slotGetKeys()
{
  if(_hobit->createJobGetKeys() == HobitErrorSuccess)
    emit send();
  else (QMessageBox::critical(this, tr("Error"), 
         tr("Could not get keys. You need not get keys for a chipcard.")));  
}

/**
* retrieves the system id of the bank server 
*/
void GuiTools::slotGetSystemId()
{
  _hobit->createJobGetSystemId();
  emit send(); 
}


/**
* sends our own public keys to the bank server 
*/
void GuiTools::slotSendKeys()
{
  if(_hobit->createJobSendKeys() == HobitErrorSuccess)
    emit send();
  else (QMessageBox::critical(this, tr("Error"), 
         tr("Could not send keys. You need not send keys for a chipcard.")));  
}

/**
* retrieves a list of our accounts 
*/
void GuiTools::slotGetAccounts()
{
  _hobit->createJobGetAccounts();
   emit send(); 
 }


/**
* generates the customers iniletter 
*/
void GuiTools::slotGenerateIniLetter()
{
  emit showIniLetter();
}

/** Run a test connection to www.google.de or another url
*/
void GuiTools::slotConnectionTest(){
 ConnectionState state;
  //connected to guiMain::slotTestInternet  
 emit testInternet(lineEditUrl->text(), lineEditPort->text().toInt(), &state); 
}

/** Start the hardware setup dialog
*/
void GuiTools::slotConsole(){
   if(! USE_LIVE_CD_FEATURES){
     QMessageBox::critical(this, tr("Error"), 
     tr("Please use your operating system to run a shell."));
     return;
   } 
   else QMessageBox::information(this, tr("Information"), 
   tr("Moneypenny will now open a root shell for hardware setup"));
     
   system("sudo xterm");
}

/** Emits signal help request.
*/
void GuiTools::slotHelp(){
 emit helpRequest("toolbox.html");
}
/** Emits signal importKey().
*/
void GuiTools::slotImportRDH2Key(){
 QString fileName = QFileDialog::getOpenFileName(                     
                    _hobit->getConfigDataDir(),
                    "RDH2 Keyfiles (*.key)",
                    this,
                    "Import RDH2 Keyfile",
                    "Choose a keyfile to import" );
 if(!fileName.isEmpty())
   emit importRDH2Key(fileName);
}

/** Emits signal sqlquery().
*/
void GuiTools::slotSQLQuery(){
  emit sqlquery();
}
/** Fires signal setShutdownMode
*/
void GuiTools::slotShutdownMode(bool autoShutdown){
 emit setShutdownMode(autoShutdown);
}
/** Emits signal back()
*/
void GuiTools::slotBack(){
  emit back();
}
/** Fires signal timeOutChanged
*/
void GuiTools::slotTimeOutChanged(){
  int newTimeOut = lineEditTimeOut->text().toInt();
  if(newTimeOut > 5) // do not allow too short times
   emit timeOutChanged(newTimeOut); 
}
/** Sets the timeOut value in lineEditTimeOut
*/
void GuiTools::setTimeOut(int newTimeOut){
 lineEditTimeOut->setText(QString::number(newTimeOut));
}
/** Saves the knoppix configuration
*/
void GuiTools::slotSaveKnoppixConfiguration(){
 emit saveKnoppixConfiguration();
}

/** Updates the data listview.
*/
void GuiTools::refresh(){
 //Do nothing
}
/** Fires signal buildPrinterDriverDatabase
*/
void GuiTools::slotBuildPrinterDriverDatabase(){
 emit buildPrinterDriverDatabase();
}

/** Emits deleteLogFiles
*/
void GuiTools::slotDeleteLogFiles(){
 emit deleteLogFiles();
}

/** set hobit object*/
void GuiTools::setHoBIT(HoBIT* hobit){
   _hobit = hobit;
}


