/***************************************************************************
                          guitransactions.h  -  description
                             -------------------
    begin                : Sa Nov 6 2004
    copyright            : (C) 2004 by Marcel Mulch
    email                : marcel@mulch-online.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef GUITRANSACTIONS_H
#define GUITRANSACTIONS_H

#include <qwidget.h>
#include <guitransactionsbase.h>
#include <qwidgetstack.h>
#include <qlayout.h>
#include <qtable.h>

#include "transaction.h"
#include "transactionitem.h"
#include "hobit.h"
#include "guimain.h"
#include "listviewitems.h"


class GuiMain;



/**
* this is the class for showing the transactions
*/
class GuiTransactions : public GuiTransactionsBase
{
  Q_OBJECT

public:

  /**
  *this is the transaction dialog constructor
  *
  *@param main a pointer on moneypenny´s main class
  *@param parent a pointer on the main widgets widget stack
  *@param name widgets title
  *@param hobit the hobit object, needed for hbci functions
  */
  GuiTransactions(QWidget *parent, GuiMain* guimain, const char *name=0, HoBIT* hobit = 0);

  /**
  *this is the transaction dialog destructor
  */
  virtual ~GuiTransactions();

/** set hobit object*/
  void setHoBIT(HoBIT* hobit){
   _hobit = hobit;
  }

  /**
  *this is a function for updating the listview depending on the selected account
  *Add a year to all toDates for "Current..." lists. There may be dates in the future.
  */
  virtual void slotChangeAccountOrDate();

  /**Builds the QTable from list.
    */
  virtual void buildTable(QTable * table, const QString dateFormat, list<Pointer<Transaction> > transactions, bool ascending);

  /**
  *this is a function for a refresh
  */
  void refresh();

  /**
  * this is a function for reading property of QPtrList<TransactionItem> _TransactionList. 
  */
  QPtrList<TransactionItem> getTransactionList()const;

  /**
  * this is a function for resizing the columns
  *
  *@param re QResizeEvent
  */
  void resizeEvent(QResizeEvent * re);
  
  /** Take care of the color for outgo transactions
  */
  void paintEvent(QPaintEvent * pe);


protected slots:

  /**
  *this is the slot for raising up the transaction widget
  */
  void slotGetTransactions();

  /**
  *this is the function which executes when the user performs a double click
  *on a list item
  *
  *@param row number of the row 
  *@param col number of the col
  *@param button number of the button
  *@param mousePos mouse-position
  */
  void slotSelectItem(int row, int col, int button, const QPoint & mousePos);

  /**
  * this is a function for activating GuiTransactionPrinter
  */
  void slotPrint();

   /** calls export Dialog */
   void slotExport();
   /** Emits helpRequest().
   */
   void slotHelpRequest();
   /** Creates a set of transaction templates from all outgoing transactions in the list.
   */
   void slotCreateTransferTemplates();
   /** Sorts by column
   */
   void slotSortByColumn(int column);
 
signals:

  void printTransactions();
  void getTransactions();
  void showDetails(Pointer <Transaction>);
  void exportSelected(QTable* table);
  void helpRequest(const QString& helpFile);
  void refreshRequest();
  void raiseWidget(int iD);

private:

  HoBIT* _hobit;
  GuiMain* _main;
  QPtrList<TransactionItem> _TransactionList;
  /** The sorting order of the transactionsTable changes on second click
   *  on a column.
   */
  int _LastColumn;
  /** Sorting direction
   */
  bool _Ascending;
  /**Format string for dates. See QDate for details
   */
   QString _DateFormat;
   /** strt date of the list
    */
   QDate _FromDate;
   /** End date of the list
    */
   QDate _ToDate;
   /** The table to be exported. May contain different formatted data
    * */
   QTable * _ExportTable; 
};

#endif
