/***************************************************************************
                          hobit_string.cpp  -  description
                             -------------------
    begin                : Mi Okt 20 2004
    copyright            : (C) 2004 by Marcel Mulch
    email                : marcel@mulch-online.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "hobit_string.h"
#include <string>
#include <stdio.h>
#include <stdarg.h>

#include <qobject.h>
#include <qregexp.h>

using namespace std;

const string HoBIT_String::format(const char *fmt, ...) {
  va_list ap;
  char *buf = 0;
  size_t size;
  string str="";

  va_start(ap,fmt);
  size = vsnprintf(buf,0,fmt,ap);
  va_end(ap);
  if (size > 0) {
    size++;
    buf=new char[size];
    va_start(ap,fmt);
    vsnprintf(buf,size,fmt,ap);
    va_end(ap);
    str=buf;
    delete[] buf;
  }
  return str;
}



/** dumps a hex string, so it can be
  * printed to the ini letter
  */
const string HoBIT_String::dumpHexString(string s) {
    string result;
    unsigned int pos;

    result+="   ";
    for (pos=0; pos < s.length(); pos++) {
        if ((pos % 40)==0)
            result+="\n   ";
        else if ((pos & 1)==0)
            result+=" ";
        result+=s.at(pos);
    } // for
    result+="\n";
    return result;
}

/** dumps a char array, so it can be
  * printed to the ini letter
  */
const string HoBIT_String::dumpHexString(char * data, int length) {
 string temp(data, length);
 return dumpHexString(temp); 
}

/**removes German Umlaute or other special chars illegal for HBCI
 * Lots of trouble with utf8, iso8859-15 and so on. Now uses unicode from linguist.
* ToDO: implement for other languages
*/
const QString  HoBIT_String::replaceSpecialChars(const QString & text){
   QString buffer;
   bool wasUpper = isUpper(text);
   for(uint pos = 0; pos < text.length(); pos ++){
    QString cPos = text[pos]; //Debugging only
    if((text[pos])== QObject::tr("aUmlaut")) 
           buffer += "ae"; 
    else if((text[pos])== QObject::tr("AUmlaut")) 
           buffer += "Ae"; 
    else if((text[pos])== QObject::tr("oUmlaut")) 
           buffer += "oe"; 
    else if((text[pos])== QObject::tr("OUmlaut")) 
           buffer += "Oe"; 
    else if((text[pos])== QObject::tr("uUmlaut")) 
           buffer += "ue"; 
    else if((text[pos])== QObject::tr("UUmlaut")) 
           buffer += "Ue"; 
    else if((text[pos])== QObject::tr("sz Char = Beta")) 
           buffer += "ss"; 
    else if((text[pos])==":")
           buffer += ".";        
    else if((text[pos])==";")
           buffer += ",";        
         else buffer+= text[pos]; 
   }//end for
   if(wasUpper)
      buffer= buffer.upper();
   return buffer;  
}
/** True, if string contains only uppercase lettters and spaces.
*/
bool HoBIT_String::isUpper(const QString & text){
  QString textUp= text.upper();
  return textUp == text;
}

/** Strips leading zeros and returns a char *
*/
QString HoBIT_String::toIdFormat(QString s){
 QString result;
 if(s== "")
    return "";
    
 uint pos = 0;
 while(s[pos] == '0') //strip leading zeros
  pos++; 
 while((s[pos] >= '0') && (s[pos] <= '9') && (pos < s.length())){ //strip currency
  result += s[pos];
  pos ++;
 } 
 return result;
}

/** Copies a QStringList to a STL list<string>
*/
void HoBIT_String::copyQStringList(QStringList * src, list <string> * dest){
  QStringList::iterator it = src->begin();
  while(it != src->end()){
   dest->push_back((*it).ascii());
   it++;
  } 
}

/** Returns true, if list is empty or contains only empty lines or whitespaces.
*/
bool HoBIT_String::stringListHasEmptyLines(QStringList list){
  bool result = list.isEmpty();
  if(! result){
   result = true;
   QStringList::iterator it = list.begin();
   while(it != list.end() && result){
     QString s = (*it).stripWhiteSpace();
     if( ! s.isEmpty())
       result = false;
     it++;  
   }//end while
  }//end if
  return result;
}
