/***************************************************************************
 *   Copyright (C) 2004 by Marcel Mulch                                    *
 *   marcel@mulch-online.de                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "hobit_transfertemplate.h"
#include "hobit_string.h"

HoBIT_TransferTemplate::HoBIT_TransferTemplate(const QString& ourAccountId,
                           const QString& otherName, 
                           const QString& otherAccountId,
                           const QString& otherBankCode,
                           double value,
                           QStringList purpose,
                           const QString& currency)
{
	_OurAccountId = ourAccountId;
	_OtherName = otherName;
	_OtherAccountId = otherAccountId;
	_OtherBankCode = otherBankCode;
	_Value = value;
        int lines = 0;
	QStringList::iterator it = purpose.begin();
        //purpose may contain illegal chars and empty lines, remove them
       while(it != purpose.end() && lines < 14){ //not more than 14 lines
         if(! (*it).isEmpty()){
           _Purpose.push_back(HoBIT_String::replaceSpecialChars((*it)));
           lines ++;
         }  
         it++; 
        }
        _Currency = currency;	
}


HoBIT_TransferTemplate::HoBIT_TransferTemplate(GWEN_DB_NODE *db) {
	fromConfig(db);
}


HoBIT_TransferTemplate::~HoBIT_TransferTemplate()
{
}


void HoBIT_TransferTemplate::fromConfig(GWEN_DB_NODE *db){
  _OurAccountId=GWEN_DB_GetCharValue(db, "ownAccountId", 0, "");
  _OtherName=GWEN_DB_GetCharValue(db, "otherName", 0, "");
  _OtherAccountId=GWEN_DB_GetCharValue(db, "otherAccountId", 0, "");
  _OtherBankCode=GWEN_DB_GetCharValue(db, "otherBankId", 0, "");
  _Value= QString(GWEN_DB_GetCharValue(db, "value/value", 0, "0,0")).toDouble();
  _Currency=  GWEN_DB_GetCharValue(db, "value/currency", 0, "EUR");
	       
  _Purpose.push_back(GWEN_DB_GetCharValue(db, "purpose/line1", 0, ""));  
  _Purpose.push_back(GWEN_DB_GetCharValue(db, "purpose/line2", 0, ""));

}



void HoBIT_TransferTemplate::toConfig(GWEN_DB_NODE *db){
   QString sValue;
   
   GWEN_DB_SetCharValue(db, GWEN_DB_FLAGS_DEFAULT,
                       "ownAccountId",
                       _OurAccountId.latin1());
   GWEN_DB_SetCharValue(db, GWEN_DB_FLAGS_DEFAULT,
                        "otherName",
                        _OtherName.latin1());
   GWEN_DB_SetCharValue(db, GWEN_DB_FLAGS_DEFAULT,
                       "otherAccountId",
                       _OtherAccountId.latin1());
   GWEN_DB_SetCharValue(db, GWEN_DB_FLAGS_DEFAULT,
                        "otherBankId",
                        _OtherBankCode.latin1());
   sValue.setNum(_Value,'f',2);                     
   GWEN_DB_SetCharValue(db, GWEN_DB_FLAGS_DEFAULT,
                        "value/value",
                        sValue.latin1());
   GWEN_DB_SetCharValue(db, GWEN_DB_FLAGS_DEFAULT,
                        "value/currency",
                        _Currency.latin1());
   
   GWEN_DB_SetCharValue(db, GWEN_DB_FLAGS_DEFAULT,
                        "purpose/line1",
                        _Purpose[0].latin1());
   GWEN_DB_SetCharValue(db, GWEN_DB_FLAGS_DEFAULT,
                        "purpose/line2",
                        _Purpose[1].latin1());
}

/** Returns true , if the minimum information is set.
*/
bool HoBIT_TransferTemplate::isValid(){
   return   (!_OtherName.isEmpty()) 
         && (!_OtherBankCode.isEmpty()) 
         && (!_OtherAccountId.isEmpty()) 
         && (!_OurAccountId.isEmpty());
}


/** All the getter methods
*/

const QString& HoBIT_TransferTemplate::ourAccountId()const
{
    return _OurAccountId;
}
const QString& HoBIT_TransferTemplate::otherName()const
{
    return _OtherName;
}
const QString& HoBIT_TransferTemplate::otherAccountId()const
{
    return _OtherAccountId;
}
const QString& HoBIT_TransferTemplate::otherBankCode()const
{
    return _OtherBankCode;
}
double HoBIT_TransferTemplate::value()const
{
    return _Value;
}
const QString& HoBIT_TransferTemplate::currency()const
{
    return _Currency;
}
const QStringList& HoBIT_TransferTemplate::purpose()const 
{
    return _Purpose;
}

